local effects = {}
effects.modifiers = {}

function effects.init()
end

function effects.drink_fountain(item)
	item_sprite = tiles.sprites[item.position][item.layer]
	local can_drink = false
	if (item.type == "Health" and tiles.player.current_health < tiles.player.health) or (item.type == "Mana" and tiles.player.current_mana < tiles.player.mana) then
		can_drink = true
	end
	if item.amount > 0 and can_drink then
		local drink_amount = 0
		if item.type == "Health" then
			drink_amount = tiles.player.health - tiles.player.current_health
		elseif item.type == "Mana" then
			drink_amount = tiles.player.mana - tiles.player.current_mana
		end
		if drink_amount > item.amount then
			drink_amount = item.amount
		end
		item.amount = item.amount - drink_amount
		if item.type == "Health" then
			tiles.player.current_health = tiles.player.current_health + drink_amount
			helper.add_message("+ "..drink_amount.." "..lang.word_health,item_sprite.position.x,item_sprite.position.y,2,"green")
		elseif item.type == "Mana" then
			tiles.player.current_mana  = tiles.player.current_mana + drink_amount
			helper.add_message("+ "..drink_amount.." "..lang.word_mana,item_sprite.position.x,item_sprite.position.y,2,"green")
		end
		if item.amount <= 0 then
			item.state = "empty"
			tiles.change_tile(item.position,item.tile_empty,tiles.image_items,item.layer, item.solid)
		end
		for item_index,olditem in ipairs(tiles.current_room.items) do
			if olditem.position == item.position then
				tiles.current_room.items[item_index] = item
			end
		end
		sounds.play_sound('drink',tiles.current_player_position)
	else
		if can_drink then
			helper.add_message(lang.msg_fountainempty,item_sprite.position.x,item_sprite.position.y,2)
		else
			if item.type == "Health" then
				helper.add_message(lang.msg_healthfull,item_sprite.position.x,item_sprite.position.y,2)
			elseif item.type == "Mana" then
				helper.add_message(lang.msg_manafull,item_sprite.position.x,item_sprite.position.y,2)
			end
		end
		sounds.fail:play() 
	end
end

function effects.drink_potion(card)
	local p_sprite = tiles.sprites[tiles.current_player_position][rooms.level.special_items.player.layer]
	local can_drink = false
	local card_amount = card.modifier.amount
	if card.modifier.property == "combat" or card.modifier.property == "movement" or (card.modifier.property == "health" and tiles.player.current_health < tiles.player.health) or (card.modifier.property == "mana" and tiles.player.current_mana < tiles.player.mana) then
		can_drink = true
	end
	if card.subtype == "Antidote" then
		for mod_index,mod in pairs(effects.modifiers) do
			if mod.amount <0 then
				player.modifiers[mod.property] = player.modifiers[mod.property]-mod.amount
				table.remove(effects.modifiers,mod_index)
				can_drink = true
			end
		end
	end
	if can_drink then
		local drink_amount = 0
		card.modifier.origin = "potion"
		if card.modifier.property == "health" then
			drink_amount = tiles.player.health - tiles.player.current_health
		elseif card.modifier.property == "mana" then
			drink_amount = tiles.player.mana - tiles.player.current_mana
		end
		if drink_amount > card.modifier.amount then
			drink_amount = card.modifier.amount
		end
		card_amount = card.modifier.amount - drink_amount
		if card.subtype == "Health" then
			tiles.player.current_health = tiles.player.current_health + drink_amount
			helper.add_message("+ "..drink_amount.." "..lang.word_health,p_sprite.position.x,p_sprite.position.y,2,"green")
		elseif card.subtype == "Mana" then
			tiles.player.current_mana  = tiles.player.current_mana + drink_amount
			helper.add_message("+ "..drink_amount.." "..lang.word_mana,p_sprite.position.x,p_sprite.position.y,2,"green")
		elseif card.subtype == "Combat" then
			effects.add_modifier("combat",card.modifier)
			card_amount = 0
			helper.add_message("+ "..card.modifier.amount.." "..lang.word_combat,p_sprite.position.x,p_sprite.position.y,2,"green")
		elseif card.subtype == "Speed" then
			effects.add_modifier("movement",card.modifier)
			card_amount = 0
			helper.add_message("+ "..card.modifier.amount.." "..lang.word_speed,p_sprite.position.x,p_sprite.position.y,2,"green")
		elseif card.subtype == "Antidote" then
			card_amount = 0
			effects.update_modifiers()
			helper.add_message(lang.msg_poison_cured,p_sprite.position.x,p_sprite.position.y,2,"green")
		end
		sounds.play_sound('drink',tiles.current_player_position)
	else
		if card.subtype == "Health" then
			helper.add_message(lang.msg_healthfull,p_sprite.position.x,p_sprite.position.y,2)
		elseif card.subtype == "Mana" then
			helper.add_message(lang.msg_manafull,p_sprite.position.x,p_sprite.position.y,2)
		elseif card.subtype == "Antidote" then
			helper.add_message(lang.msg_nopoison,p_sprite.position.x,p_sprite.position.y,2)
		end
		sounds.fail:play() 
	end
	return card_amount
end

function effects.cast_spell(card,scroll)
	local p_sprite = tiles.sprites[tiles.current_player_position][rooms.level.special_items.player.layer]
	local effect = card.effect
	if scroll then
		effect = card.modifier
	end
	if player.current_mana >= effect.cost then
		if effect.type == "action" then
			if effect.action == "teleport" then
				actions.current_mode = "teleport"
			elseif effect.action == "bolt" then
				if scroll then
					actions.current_mode = "bolt_scroll"
				else
					actions.current_mode = "bolt"
				end
			elseif effect.action == "magicopen" then
				actions.current_mode = "magicopen"			
			elseif effect.action == "showtraps" then
				actions.show_traps()
			end
		elseif effect.type == "modifier" then
			local can_cast = false
			local err_msg = lang.msg_casterror
			local card_amount = effect.amount
			if effect.property == "combat" or effect.property == "movement" or effect.property == "stealth"  or effect.property == "robustness" then
				can_cast = true
			elseif effect.property == "current_health" then
				if player.current_health < player.health then 
					can_cast = true
				else
					err_msg = lang.msg_healthfull
					can_cast = false
				end
			elseif effect.property == "current_mana" then
				if player.current_health > effect.amount_out and player.current_mana < player.mana then
					can_cast = true
				else
					can_cast = false
					if player.current_health < 2 then
						err_msg = lang.msg_healthtoolow
					elseif player.current_mana == player.mana then
						err_msg = lang.msg_manafull
					end
				end
			end
			helper.log(effect.property.." : "..player.current_health.." -> "..player.current_mana.." / "..player.mana)
			if can_cast then
				local cast_amount = 0
				effect.spell = card.name
				if effect.property == "current_health" then
					cast_amount = tiles.player.health - tiles.player.current_health
				elseif effect.property == "current_mana" then
					cast_amount = tiles.player.mana - tiles.player.current_mana
				end
				if cast_amount > effect.amount then
					cast_amount = effect.amount
				end
				card_amount = effect.amount - cast_amount
				if effect.property == "current_health" then
					tiles.player.current_health = tiles.player.current_health + cast_amount
					helper.add_message("+ "..cast_amount.." "..lang.word_health,p_sprite.position.x,p_sprite.position.y,2,"green")
				elseif effect.property == "current_mana" then
					tiles.player.current_mana  = tiles.player.current_mana + cast_amount
					player.current_health = player.current_health - effect.amount_out
					helper.add_message("+ "..cast_amount.." "..lang.word_mana,p_sprite.position.x,p_sprite.position.y,2,"green")
				elseif effect.property == "combat" then
					effects.add_modifier("combat",effect)
					helper.add_message("+ "..effect.amount.." "..lang.word_combat,p_sprite.position.x,p_sprite.position.y,2,"green")
				elseif effect.property == "movement" then
					effects.add_modifier("movement",effect)
					helper.add_message("+ "..effect.amount.." "..lang.word_speed,p_sprite.position.x,p_sprite.position.y,2,"green")
				elseif effect.property == "stealth" then
					effects.add_modifier("stealth",effect)
					helper.add_message("+ "..effect.amount.." "..lang.word_stealth,p_sprite.position.x,p_sprite.position.y,2,"green")
				elseif effect.property == "robustness" then
					effects.add_modifier("robustness",effect)
					helper.add_message("+ "..effect.amount.." "..lang.word_robustness,p_sprite.position.x,p_sprite.position.y,2,"green")
				end
				sounds.play_sound('cast',tiles.current_player_position)
				player.current_mana =player.current_mana - effect.cost
			else
				helper.add_message(err_msg,p_sprite.position.x,p_sprite.position.y,2)
				sounds.fail:play() 
			end
		end
	else
		helper.add_message(lang.msg_notenoughmana,p_sprite.position.x,p_sprite.position.y,2)
		sounds.fail:play() 
	end
	return card_amount
end

function effects.add_modifier(property,modifier)
	player.modifiers[property] = player.modifiers[property] + modifier.amount
	modifier.current_duration = modifier.duration
	table.insert(effects.modifiers,modifier)
end

function effects.update_modifiers()
	for mod_index,mod in pairs(effects.modifiers) do
		mod.current_duration = mod.current_duration -1
		helper.log("Property : "..mod.property)
		helper.log("Duration : "..mod.duration)
		helper.log("Time : "..mod.current_duration)
		if mod.current_duration < 1 then
			player.modifiers[mod.property] = player.modifiers[mod.property] - mod.amount
			table.remove(effects.modifiers,mod_index)	
			helper.debug = ""		
			helper.log("Property : "..mod.property)
			helper.log("New value : ".. player.modifiers[mod.property])
		end
	end
	for mod_index,mod in pairs(player.delayed_modifiers) do
		if player["current_"..mod_index]<player[mod_index] then
			mod.current_delay = mod.current_delay -1
			helper.log("Regénération "..mod_index.." : "..mod.current_delay)
			local ratio = mod.current_delay/mod.delay
			interface.delays[mod_index] = 100-math.floor(100*ratio)
			helper.log(interface.delays.health)
			if mod.current_delay == 0 then
				player["current_"..mod_index] = player["current_"..mod_index] + mod.amount
				mod.current_delay = mod.delay
			end
		else
			interface.delays[mod_index] = nil
		end
	end
	sheet.drawsheet()
end

function effects.update_item_modifiers()
	local item_modifiers = {movement = 0, combat = 0, vision = 0}
	local modifiers = {}
	for item_index,item in pairs(player.items) do
		if item.modifier and item.modifier.type == "equip" then
			local item_id = item.type.."."..item.subtype
			print(item_id,helper.count_occurrences(modifiers,item_id))
			if player.ambidextrous and item.modifier.ambi then
				item.modifier.max = 2
				print("Max : ",item.modifier.max)
			end
			if item.modifier.max == 0 or helper.count_occurrences(modifiers,item_id)<item.modifier.max then
				item_modifiers[item.modifier.property] = item_modifiers[item.modifier.property]+item.modifier.amount
				table.insert(modifiers,item_id)
			end
		end
	end
	for mod_index,mod in pairs(item_modifiers) do
		player.item_modifiers[mod_index] = mod
	end
	print("Modifier vision",item_modifiers.vision)
	print("Modifier movement",item_modifiers.movement)
	sheet.drawsheet()
end

function effects.alert_npcs(alert_level)
	for _, npc in pairs( tiles.current_room.npcs ) do
		npc.vigilance = npc.vigilance + alert_level
	end
end

function effects.use_orb(card)
	local p_sprite = tiles.sprites[tiles.current_player_position][rooms.level.special_items.player.layer]
	local ahead_tile = path.get_ahead_tile(tiles.current_player_position,tiles.player_direction)
	local icon = interface.icon_tiles.fail
	local sound = sounds.fail
	for item_index, item in pairs(tiles.current_room.items) do
		if item.type == "Mechanism" and item.orbs_count<item.orbs_max and item.position == ahead_tile then
			local adjacent = path.get_adjacent(tiles.current_player_position,false,true,true)
	      	local is_adjacent = helper.in_array(adjacent,item.position)
			if is_adjacent then
				table.remove(interface.cards,card.index)
				item.orbs_count = item.orbs_count + 1
				item.tile = item.tile_steps[item.orbs_count]
				tiles.change_tile(item.position,item.tile,tiles.image_items,item.layer,item.solid)
				if item.orbs_count == item.orbs_max then
					local event = effects.start_event(item.action)
					icon = interface.icon_tiles.mechanism_end
					sound = sounds.mechanism_end
				else
					icon = interface.icon_tiles.mechanism_step
					sound = sounds.mechanism_step
				end
			end
		end
	end
	interface.show_icon(icon)
	sound:play() 
end

function effects.start_event(event_name)
	local result = false
	if levels.current.events[event_name] then
		for event_index, event in pairs(levels.current.events[event_name]) do
			if event.targets.create then
				if event.targets.create.exit then
					effects.create_exit(event)
				elseif event.targets.create.finalboss then
					effects.create_boss(event)
				end
			end
		end
	end
	return result
end

function effects.create_exit(event)
	local exit = rooms.set_exit_location(event.targets.create.exit)
	local room_index = exit.room.x..exit.room.y
	helper.log("SORTIE :::: "..exit.room.x.." - "..exit.room.y)
	local room
	if not rooms.map[room_index] then
		room = rooms.add_room(room_index,"exit",nil,exit.room)
	else
		room = rooms.map[room_index].room
	end	
	local wall = room.walls[exit.orientation]
	wall[exit.door_position] = exit.tile
	rooms.map[room_index].room.exit = exit
	rooms.map[room_index].room.walls[exit.orientation] = wall	
	if event.targets.create.message then
		menu.alertmenu(event.targets.create.message,"menu_continue","hint")
	end
	if event.targets.create.compass then
		interface.init_compass(exit.room,nil,nil,'main')
		interface.update_compass()
	end
end

function effects.create_boss(event) 
	-- set boss location
	local boss_room = rooms.get_random_room("only")
	if not boss_room then
		boss_room =  rooms.get_random_room("include")
	end
	local room_index = rooms.coords_index(boss_room)
	-- set boss room	
	local room
	if not rooms.map[room_index] then
		room = rooms.add_room(room_index)
	else
		room = rooms.map[room_index].room
	end	
	-- set boss
	local tmpboss = rooms.level.boss[event.targets.create.boss[1]]
	local boss = {}	    
    for k, v in pairs (tmpboss) do
	  boss[k] = v
	end
	boss.direction = path.get_random_direction()
	local placeable = nil
	if boss.placeable then
		placeable = boss.placeable
	end
	boss.position = rooms.set_item_position(room,"npc",placeable)
    table.insert(room.npcs,boss)
	if event.targets.create.compass then
		interface.init_compass(boss_room,nil,nil,'main')
		interface.update_compass()
	end
	if event.targets.create.message then
		menu.alertmenu(event.targets.create.message,"menu_continue","hint")
	end
end

return effects