local home = {}
binser = require "helpers/binser"
config = require "config"
local configsaved = love.filesystem.getInfo("conf")
local newconfig
if configsaved then
   local conf = love.filesystem.read("string","conf")
   newconfig = binser.deserializeN(conf)
   for var,val in pairs(config) do
      if newconfig[var] and var~="version" then
         config[var] = newconfig[var]
      end
   end
end
lang = require "lang"
lang.init (config.lang)
helper = require "helpers/helper"
interface = require "interface"
interface.init()
menu = require "menu"
menu.init('home')
menu.music = love.audio.newSource("sounds/home.ogg", "stream")
menu.update_music_volume()
cursor_default = love.mouse.newCursor( "images/1200x780/cursor_1.png", 10, 1 )
interface.setCursor(cursor_default)
-- shader test
brightness_shader = love.graphics.newShader([[
   extern number brightness;
   vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
   {
      vec4 texcolor = Texel(texture, texture_coords);
      vec4 result = vec4(texcolor.rgb * (brightness+0.4), texcolor.a);
      return result * color;
   }
]])

function home.load()
end

function home.update( dt )
   if not menu.music:isPlaying( ) then
      love.audio.play( menu.music )
   end
end

function home.draw() 
   love.graphics.setShader(brightness_shader) 
   brightness_shader:send("brightness", config.brightness)
   love.graphics.draw( interface.image_home , 0, 0, 0, love.graphics.getWidth()/interface.image_home:getWidth(),love.graphics.getHeight()/interface.image_home:getHeight())
   -- menu
   local start_x = love.graphics.getWidth()/2 -menu.current_params.width/2
   local start_y =  love.graphics.getHeight()/2 - menu.current_params.height/2
   love.graphics.draw(menu.canvas,start_x,start_y)
   local start_y =  love.graphics.getHeight()-40
   love.graphics.printf({interface.font_grey,"Version "..config.version}, interface.font_medium, 0, start_y,love.graphics.getWidth(),'center')
end

function home.keyreleased( key, code )
   if key == "return" then
      gamestates.set_state( "game", { current_room = 1 } )
   elseif key == 'escape' then
      love.event.quit()
   end       
   if key == 'f' then
         if love.window.getFullscreen() then
            love.window.setFullscreen(false)
         else
            love.window.setFullscreen(true)   
         end         
   end   
end

function love.mousemoved(x, y, button)
   local start_x = love.graphics.getWidth()/2 -menu.current_params.width/2
   local start_y =  love.graphics.getHeight()/2 - menu.current_params.height/2
   local rel_x = x - start_x
   local rel_y = y - start_y
   interface.setCursor(interface.cursor_default)
   for k,item in ipairs(menu.current_items) do 
      if item.action and rel_x>item.x and rel_x<item.x+item.width and rel_y>item.y and rel_y<item.y+item.height then
         menu.drawmenu()
         item.hover = true
         interface.setCursor(interface.cursor_hand)
      else
         item.hover = false
      end
   end
end

function love.mousepressed(x, y, button)
   local start_x = love.graphics.getWidth()/2 -menu.current_params.width/2
   local start_y =  love.graphics.getHeight()/2 - menu.current_params.height/2
   local rel_x = x - start_x
   local rel_y = y - start_y
   
   for k,item in ipairs(menu.current_items) do 
      if item.action then
         if item.action.task == "setting" then
            for _,button in ipairs(menu.setting_buttons) do 
               if rel_x>button.x and rel_x<button.x+item.width and rel_y>button.y+button.height/2 and rel_y<button.y+button.height+button.height/2 then
                  menu.action(item.action,button.prop,button.value)
                  menu.drawmenu()
               end
            end
         elseif rel_x>item.x and rel_x<item.x+item.width and rel_y>item.y and rel_y<item.y+item.height then
            menu.action(item.action)
         end
      end
   end
end

return home
