local levels = {}
levels.current_level = 1

function levels.require_current_level()
   -- local room_filename = "levels/rooms/"..rooms.current_room
   -- local room = require( room_filename )
   local level_filename = "levels/"..levels.current_level.."/level.lua"
   -- use load instead of require to reload file on restart
   print("Fichier : "..tostring(level_filename))
   local chunk = love.filesystem.load( level_filename )
   local level = chunk()
   level.tile_items_deck = levels.build_deck(level.tile_items,"tile_items")
   level.tile_npcs_deck = levels.build_deck(level.tile_npcs,"tile_npcs")
   level.card_items_deck = levels.build_deck(level.card_items,"card_items")
   levels.current = level
   return level
end

function levels.build_deck(cards,card_type)
	deck = {}
	for _, card in pairs( cards ) do 
		for y=1, card.count do
			card = levels.format_card(card, card_type)
			table.insert(deck,card)
		end
	end
	deck = levels.shuffle(deck)
	return deck
end

function levels.format_card(card,card_type)
	-- card defaults
	if card_type == "card_items" then
		if card.modifier then
			if not card.modifier.max then
				card.modifier.max = 0
			end
			if not card.modifier.duration then
				card.modifier.duration = 0
			end
			if not card.modifier.amount then
				card.modifier.amount = 0
			end
			if not card.modifier.cost then
				card.modifier.cost = 0
			end
		end
	end
	return card
end

function levels.shuffle(deck,insert_index)
	shuffled = {}
	for i, v in pairs(deck) do
		local pos = math.random(1, #shuffled+1)
		if insert_index then
			v.index = i
		end
		table.insert(shuffled, pos, v)
	end
	return shuffled
end

function levels.get_card_by_type(deck,type,id)
	for i,card in pairs(deck) do
		if card.type == type then
			if id == nil or id == card.id then
				table.remove(deck,i)
				return card
			end
		end
	end
	return false
end

return levels