local level = {}
level.size = {
   height = 4,
   width = 5
}
level.start_type = "edge"
level.start_position = "random"
level.start_room = {}
level.player_start_position = nil
level.entrance = {tiles={NS="07",WE="17"},player_position="random"}
level.tilesets = {
   floor = "image_floor",
   wall = "image_wall",
   items = "image_items",
   npcs = "image_creatures"
}
level.music = "game5.ogg"
level.cell_model = "indoor"
level.cell_open_ratio = 0.25
level.trap_ratio = 0.5
level.trap_count = 2
level.cell_lock_level = 1
level.chest_lock_level = 3
level.trap_deactivation_level = 2
level.items_count = 2
level.player_start_position = 25
level.min_npcs = 1
level.max_npcs = 4
level.num_layers = 4
level.theme = "temple"
level.reset = {
   player = {
      orbs=0,
      modifiers={movement=0,combat=0,stealth=0,robustness=0,vision=0}
   }
}
level.skills = {"spells","abilities"}
level.events = {
   create_room = {
      {
         state = 'pending',
         count = {min=8,max=10},
         main = true,
         targets = {
            create = {
               items = { "mechanism" },
               boss = {"hezrou"},
               compass = true,
               cells = {cells = {C = true}, open_ratio = 0},
               active = {items = true, boss = true, cells = true, compass = true, message = true}
            }
         }
      },
      {
         state = 'pending',
         count = {min=4,max=5},
         main = true,
         targets = {
            create = {
               items = { "health" },
               cells = {cells = {C = true}, open_ratio = 0},
               active = {items = true, cells = true, message = true}
            }
         }
      },
      {
         state = 'pending',
         count = {min=6,max=7},
         main = true,
         targets = {
            create = {
               items = { "mana" },
               cells = {cells = {C = true}, open_ratio = 0},
               active = {items = true, cells = true, message = true}
            }
         }
      }
   },
   create_exit = {
      {
         state = 'active',
         targets = {
            create = {
               exit = {tiles={NS="08",WE="18"},room_position='random',door_position='random',room={}},
               boss = {"skeleton","skeleton","skeleton"},
               compass = true,
               active = {exit = true, boss = true, message = true},
               message = "hint_level2_exit"
            }
         }
      }
   }
}
level.objectives = {
   objective = "pit",
   condition = "orbs",
   has_boss = true,
   count_min = 10,   
   count_max = 15,    
   starttext = "level_start_5",
   endtext = "level_end_5",
   endgame = true
}
level.conditions = {
   items = {
      {
         attr = 'type',
         val = 'Mechanism',
         min = 1
      }
   }
}
level.tile_items = {
   { 
      type="Chest",
      tile="00",
      tile_opened="10",
      max_gold=3,
      max_cellkey=2,
      max_chestkey=0,
      max_items=2,
      solid=true,
      layer=3,
      count=24,
      action="pick"
   },
   { 
      type="Barrel",
      subtype="Barrel",
      tile="07",
      tile_opened="17",
      max_gold=3,
      max_cellkey=1,
      max_chestkey=0,
      max_items=0,
      solid=true,
      layer=3,
      count=24,
      action="pick"
   },
   { 
      type="Barrel",
      subtype="Crate",
      tile="08",
      tile_opened="18",
      max_gold=1,
      max_cellkey=1,
      max_chestkey=0,
      max_items=0,
      solid=true,
      layer=3,
      count=24,
      action="pick"
   },
   { 
      type="Barrel",
      subtype="Crate",
      tile="09",
      tile_opened="19",
      max_gold=1,
      max_cellkey=1,
      max_chestkey=1,
      max_items=0,
      solid=true,
      layer=3,
      count=12,
      action="pick"
   },
   { 
      type="Health",
      tile="01",      
      tile_empty="11",
      solid=true,
      amount=5,
      layer=3,
      count=8,
      action="pick"
   },
   { 
      type="Mana",
      tile="02",      
      tile_empty="11",
      solid=true,
      amount=5,
      layer=3,
      count=8,
      action="pick"
   },
   { 
      type="Altar",
      tile="03",
      solid=true,
      layer=3,
      count=0,
      action="pick"
   },
   { 
      type="Portal",
      id="1",
      tile="04",
      solid="npcs",
      layer=1,
      count=2,
      action="teleport",
      sparkle=0.5
   },
   { 
      type="Portal",
      id="2",
      tile="05",
      solid="npcs",
      layer=1,
      count=2,
      action="teleport",
      sparkle=0.2
   },
   { 
      type="Portal",
      id="3",
      tile="06",
      solid="npcs",
      layer=1,
      count=2,
      action="teleport",
      sparkle=0.2
   }
}
level.special_items = {
   player =
   { 
      type="Player",
      tile="00",
      tileset="image_creatures",
      solid=true,
      layer=3,
   },
   purse =
   { 
      type="Purse",
      tile="20",
      solid=false,
      layer=2,
      booty={},
      action="pick"
   },
   halo =
   { 
      type="Halo",
      tile="21",
      tileset="image_items",
      solid="npcs",
      layer=4
   },
   pit =
   { 
      type="Pit",
      tile="22",
      tileset="image_items",
      placeable = "cell",
      solid="npcs",
      layer=1
   },
   trap =
   { 
      type="Trap",
      tile="15",
      tile_triggered="25",
      tileset="image_items",
      placeable = "hall",
      solid = "npcs",
      damage=1,
      layer=4
   },
   mechanism =
   { 
      type="Mechanism",
      subtype="Green",
      tile="14",
      tileset="image_items",
      tile_steps={"24"},
      orbs_count=0,
      orbs_max=1,
      placeable = "cell",
      solid=true,
      layer=3,
      position=25,
      action="create_exit"
   },
   portal =
   { 
      type="Portal",
      id="4",
      tile="04",
      solid="npcs",
      layer=1,
      action="teleport"
   },
   chest=
   { 
      type="Chest",
      tile="00",
      tile_opened="10",
      max_gold=3,
      max_cellkey=2,
      max_chestkey=0,
      max_items=2,
      solid=true,
      layer=3,
      action="pick"
   },
   health=
   { 
      type="Health",
      tile="01",      
      tile_empty="11",
      solid=true,
      amount=5,
      layer=3,
      action="pick"
   },
   mana=
   { 
      type="Mana",
      tile="02",      
      tile_empty="11",
      solid=true,
      amount=5,
      layer=3,
      action="pick"
   },
   bolt =
   { 
      type="Bolt",
      tile="26",
      tileset="image_items",
      solid="npcs",      
      placeable = "cell",
      layer=4
   }
}
level.tile_npcs = {
   { 
      type="Lizardfolk",
      tile="01",
      solid=true,
      speed=4,
      health=2,
      attack=1,
      defense=2,
      count=24,
      max_gold=1,
      max_chestkey=0,
      max_cellkey=1,
      max_items=1,
      layer=3,
      hostility = 1,
      vigilance = 3
   },
   { 
      type="Hobgoblin",
      tile="02",
      solid=true,
      speed=3,
      health=2,
      attack=2,
      defense=2,
      count=16,
      max_gold=2,
      max_chestkey=1,
      max_cellkey=2,
      max_items=1,
      layer=3,
      placeable = "hall",
      hostility = 1,
      vigilance = 4
   },
   { 
      type="Scorpion",
      tile="04",
      solid=true,
      speed=5,
      health=2,
      attack=1,
      defense=2,
      count=24,
      max_gold=0,
      max_chestkey=0,
      max_cellkey=0,
      max_items=0,
      layer=3,
      placeable = "all",
      hostility = 1,
      vigilance = 3,
      poison = {property="vision",amount=-1,repetition=0,duration=10,level=2}
   },
   { 
      type="Dragonborn",
      tile="05",
      solid=true,
      speed=4,
      health=2,
      attack=1,
      defense=2,
      count=16,
      max_gold=1,
      max_chestkey=0,
      max_cellkey=1,
      max_items=1,
      layer=3,
      hostility = 1,
      vigilance = 3
   }
}
level.boss = {
   hezrou = { 
      type="Hezrou",
      tile="07",
      solid=true,
      speed=4,
      health=3,
      attack=4,
      defense=3,
      count=1,
      max_gold=10,
      max_chestkey=1,
      max_cellkey=1,
      max_items=3,
      layer=3,
      placeable = "hall",
      hostility = 1,
      vigilance = 3
   }
}
level.card_items = {   
   { 
      type="potion",
      subtype="Speed",
      name="card_title_speed",
      desc="card_desc_speed",
      tile="03",
      amount=2,
      value=8,
      uses=1,
      count=8,
      modifier = {type="timed",property="movement",amount=2,duration=10,max=0}
   },
   {  
      type="potion",
      subtype="Health",
      name="card_title_health",
      desc="card_desc_health",
      tile="00",
      value=8,
      uses=1,
      count=8,
      modifier = {type="unique",property="health",amount=1,duration=0}
   },
   { 
      type="potion",
      subtype="Combat",
      name="card_title_combat",
      desc="card_desc_combat",
      tile="01",
      value=8,
      uses=1,
      count=8,
      modifier = {type="timed",property="combat",amount=1,duration=10,max=0}
   },
   { 
      type="potion",
      subtype="Mana",
      name="card_title_mana",
      desc="card_desc_mana",
      tile="02",
      value=8,
      uses=1,
      count=8,
      modifier = {type="unique",property="mana",amount=1,duration=0}
   },
   { 
      type="potion",
      subtype="Antidote",
      name="card_title_antidote",
      desc="card_desc_antidote",
      tile="04",
      value=8,
      uses=1,
      count=8,
      modifier = {type="unique",property=nil,amount=1,duration=0}
   },
   { 
      type="orb",
      subtype="Green",
      name="card_title_greenorb",
      desc="card_desc_orb",
      tile="10",
      value=8,
      uses=1,
      count=6,
      modifier = {type="equip",property="movement",amount=-1}
   },
   { 
      type="weapon",
      subtype="Sword",
      name="card_title_sword",
      desc="card_desc_sword",
      tile="11",
      value=8,
      uses=0,
      count=1,
      modifier = {type="equip",property="combat",amount=1,max=1,ambi=true}
   },
   { 
      type="clothe",
      subtype="Speedboots",
      name="card_title_speedboots",
      desc="card_desc_speedboots",
      tile="12",
      value=8,
      uses=0,
      count=1,
      modifier = {type="equip",property="movement",amount=1,max=1}
   },
   { 
      type="torch",
      subtype="Torch",
      name="card_title_torch",
      desc="card_desc_torch",
      tile="14",
      value=4,
      uses=0,
      count=8,
      modifier = {type="equip",property="vision",amount=2,max=1}
   },
   { 
      type="scroll",
      subtype="OPENING",
      name="card_title_scroll",
      desc="card_desc_scroll_OPENING",
      tile="13",
      value=4,
      uses=0,
      count=6,
      modifier = {type="action",action="magicopen",amount=1,duration=0,cost=0}
   },
   { 
      type="scroll",
      subtype="TRAPDETECTION",
      name="card_title_scroll",
      desc="card_desc_scroll_TRAPDETECTION",
      tile="13",
      value=4,
      uses=0,
      count=6,
      modifier = {type="action",action="showtraps",amount=1,duration=1,cost=0}
   }
}
return level