local map = {}

map.visible = false
map.set = false
map.roomsize = 52
map.doorsize = 7
map.cellsize = 22
map.margin = 15
map.minwidth = love.graphics.getWidth()/4
map.options = {
   show_cell_opened = false
}
map.size = {}

function map.init()
   if not map.set then
      map.width = map.roomsize * rooms.level.size.width + map.margin*2
      if map.width < map.minwidth then
         map.width = map.minwidth
      end
      map.height = love.graphics.getHeight() 
      map.size = map.get_size()
      map.x = love.graphics.getWidth()
      map.y = 0
      map.x_in = love.graphics.getWidth()-map.width
      map.x_out = map.x
      map.canvas = love.graphics.newCanvas(map.width, map.height)
      map.sliding = true
      map.set = true
   end
   map.drawmap()
end

function map.showmap()
   if map.visible then
      interface.buttons.map.state = "default"
      map.sliding = "out"
   else
      map.visible = true
      interface.buttons.map.state = "active"
      map.sliding = "in"
   end
   interface.buttons.map.prevstate = interface.buttons.map.state
end

function map.drawmap()
   --local font = love.graphics.newFont(20)
   --love.graphics.setFont(font)
   love.graphics.setCanvas(map.canvas)
   love.graphics.clear()
   love.graphics.setBlendMode("alpha")  
   -- background
   love.graphics.setColor(0.1, 0.1, 0.1, 0.6)
   love.graphics.rectangle('fill', 0, 0, map.width, map.height)
   map.size = map.get_size()
   local ybase = (map.height - map.size.height - map.margin*2)/2
   local xbase = (map.width - map.size.width - map.margin*2)/2

   local current_room = rooms.current_room_location.x..rooms.current_room_location.y
   for room_index,room in pairs(rooms.map) do
      if room.room.visited or helper.mode == "dev" then
         local x = string.sub(room_index,1,1)
         local y = string.sub(room_index,2)
         xroom = x*map.roomsize - map.roomsize + xbase
         yroom = y*map.roomsize - map.roomsize + ybase
         --if room_index == current_room then   
         --   love.graphics.setColor(0.2, 0.8, 0.2, 0.8)
         --else
            love.graphics.setColor(1, 1, 0.8, 0.8)
         -- end
         if not room.room.visited then
            love.graphics.setColor(0.3, 0.3, 0.3, 0.8)
         end
         -- start room
         if rooms.level.start_room.x..rooms.level.start_room.y == room_index then
            love.graphics.setColor(0.2, 0.2, 0.8, 0.8)
         end
         -- exit room
         if room.exit then
            helper.log("rooms.exit")
            if rooms.coords_index(room.exit.room) == room_index then
               love.graphics.setColor(1, 0.4, 0.2, 0.8)
            end
         end
         love.graphics.rectangle('line', xroom, yroom, map.roomsize, map.roomsize)
         -- doors
         for direction,doors in pairs(room.room.map_doors) do
            local xdoor = 0
            local ydoor = 0
            if direction == "E" then
               xdoor = map.roomsize
            end
            if direction == "S" then
               ydoor = map.roomsize
            end
            for door_index,door in pairs(doors) do
               xdoor = xroom + xdoor - map.doorsize/2
               ydoor = yroom + ydoor - map.doorsize/2
               if direction == "E" or direction == "W" then
                  ydoor = ydoor + door * map.doorsize - map.doorsize/2
               end
               if direction == "N" or direction == "S" then
                  xdoor = xdoor + door * map.doorsize - map.doorsize/2
               end
               love.graphics.rectangle('line', xdoor, ydoor, map.doorsize, map.doorsize)
            end
         end
         -- cells
         for _,cell in pairs(room.room.cells) do
            if cell.seen then
               if map.options.show_cell_opened then
                  local open_text = ""
                  local light_text = ""
                  if cell.opened then
                     open_text = "O"
                  else            
                     open_text = "F"
                  end
                  if cell.lighted then
                     light_text = "A"
                  else            
                     light_text = "E"
                  end
               end
               if cell.position == "NW" and cell.seen then
                  love.graphics.rectangle('line', xroom, yroom, map.cellsize, map.cellsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom+5, yroom+5)
                     love.graphics.print(light_text, xroom+15, yroom+5)
                  end
               elseif cell.position == "N" then
                  love.graphics.rectangle('line', xroom, yroom, map.roomsize, map.cellsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom+5, yroom+5)
                     love.graphics.print(light_text, xroom+15, yroom+5)
                  end
               elseif cell.position == "W" then
                  love.graphics.rectangle('line', xroom, yroom, map.cellsize, map.roomsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom+5, yroom+5)
                     love.graphics.print(light_text, xroom+15, yroom+5)
                  end
               elseif cell.position == "NE" then
                  love.graphics.rectangle('line', xroom + map.cellsize + map.doorsize, yroom, map.cellsize, map.cellsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom + map.cellsize + map.doorsize+5, yroom+5)
                     love.graphics.print(light_text, xroom + map.cellsize + map.doorsize+15, yroom+5)
                  end
               elseif cell.position == "E" then
                  love.graphics.rectangle('line', xroom + map.cellsize + map.doorsize, yroom, map.cellsize, map.roomsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom + map.cellsize + map.doorsize+5, yroom+5)
                     love.graphics.print(light_text, xroom + map.cellsize + map.doorsize+15, yroom+5)
                  end
               elseif cell.position == "S" then
                  love.graphics.rectangle('line', xroom, yroom + map.cellsize + map.doorsize, map.roomsize, map.cellsize) 
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom+5, yroom + map.cellsize + map.doorsize+5) 
                     love.graphics.print(light_text, xroom+15, yroom + map.cellsize + map.doorsize+5)    
                  end       
               elseif cell.position == "SW" then
                  love.graphics.rectangle('line', xroom, yroom + map.cellsize + map.doorsize, map.cellsize, map.cellsize) 
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom+5, yroom + map.cellsize + map.doorsize+5) 
                     love.graphics.print(light_text, xroom+15, yroom + map.cellsize + map.doorsize+5)     
                  end             
               elseif cell.position == "SE" then
                  love.graphics.rectangle('line', xroom + map.cellsize + map.doorsize, yroom + map.cellsize + map.doorsize, map.cellsize, map.cellsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, xroom + map.cellsize + map.doorsize+5, yroom + map.cellsize + map.doorsize+5)  
                     love.graphics.print(light_text, xroom + map.cellsize + map.doorsize+15, yroom + map.cellsize + map.doorsize+5)    
                  end 
               elseif cell.position == "C" then
                  love.graphics.rectangle('line', xroom + (map.roomsize-map.cellsize)/2, yroom + (map.roomsize-map.cellsize)/2, map.cellsize, map.cellsize)
                  if map.options.show_cell_opened then
                     love.graphics.print(open_text, (xroom + (map.roomsize-map.cellsize))/2+5, (yroom + (map.roomsize-map.cellsize))/2+5)  
                     love.graphics.print(light_text, (xroom + (map.roomsize-map.cellsize))/2+15, (yroom + (map.roomsize-map.cellsize))/2+5)  
                  end 
               end
            end
         end         
         -- player position
        if room_index == current_room then
           love.graphics.setColor(0.5, 1, 0.5, 0.8)
           local pos = path.get_coordinates(tiles.current_player_position)
           if pos.x then
              pos.x = 8 - pos.x
              local xplayer = xroom + pos.x*map.doorsize - map.doorsize
              local yplayer = yroom + pos.y*map.doorsize - map.doorsize
              -- love.graphics.circle('fill', xplayer, yplayer, map.doorsize/2)
              love.graphics.print("@",interface.font_small,xplayer,yplayer)
            end
         end
         -- npcs (debug)
         if helper.mode == "dev" then
            love.graphics.setColor(0.8, 0.2, 0.2, 0.8)
            for _,npc in pairs(room.room.npcs) do
              local pos = path.get_coordinates(npc.position)
              if pos.x then
                 pos.x = 8 - pos.x
                 local xnpc = xroom + pos.x*map.doorsize - map.doorsize/2
                 local ynpc = yroom + pos.y*map.doorsize - map.doorsize/2
                 love.graphics.circle('fill', xnpc, ynpc, map.doorsize/2,5)
               end
            end
         end
      end
   end
   love.graphics.setCanvas()
end

function map.get_size()
   local xrooms = {}
   local yrooms = {}
   for x=1,rooms.level.size.width do
      for y=1,rooms.level.size.height do
         -- if empty room
         if rooms.map[x..y] ~= nil then
            room = rooms.map[x..y]
            if room.room.visited or helper.mode == "dev" then
               table.insert(xrooms,x)
               table.insert(yrooms,y)
            end
         end
      end
   end
   table.sort(xrooms)
   table.sort(yrooms)
   local width = (xrooms[#xrooms]-xrooms[1]) * map.roomsize + map.roomsize + map.margin*3
   local height = (yrooms[#yrooms]-yrooms[1]) * map.roomsize + map.roomsize + map.margin*3
   local x = xrooms[1] * map.roomsize - map.roomsize - map.margin
   local y = yrooms[1] * map.roomsize - map.roomsize - map.margin
   return {width=width,height=height,x=x,y=y}
end

return map