local player = {
	movement = 5,
	last_movement = {},
	cellkeys = 0,
	chestkeys = 0,
	gold = 0,
	orbs = 0,
	health = 5,
	current_health = 5,
	mana = 0,
	current_mana = 0,
	combat = 6,
	vision = 5,
	stealth = 0,
	dodge = 0,
	robustness = 0,
	max_items = 5,
	modifiers = {
		movement = 0,
		combat = 0,
		stealth = 0,
		robustness = 0,
		vision = 0
	},
	item_modifiers = {
		movement = 0,
		combat = 0,
		vision = 0,
		stealth = 0
	},
	delayed_modifiers = {
	},
	items = {},
	spells = {},
	skills = {
		paths = {},
		attributes = {},
		abilities = {},
		spells = {}
	},
	defaults = {
		resistance = 3
	}
}

function player.reset()
	player.movement = 5
	player.last_movement = {}
	player.cellkeys = 0
	player.chestkeys = 0
	player.gold = 0
	player.orbs = 0
	player.health = 5
	player.current_health = 5
	player.mana = 0
	player.current_mana = 0
	player.combat = 6
	player.vision = 5
	player.stealth = 0
	player.dodge = 0
	player.robustness = 0
	player.max_items = 5
	player.modifiers = {
		movement = 0,
		combat = 0,
		stealth = 0,
		robustness = 0,
		vision = 0
	}
	player.item_modifiers = {
		movement = 0,
		combat = 0,
		vision = 0
	}
	player.delayed_modifiers = {
	}
	player.items = {}
	player.spells = {}
	player.skills = {
		paths = {},
		attributes = {},
		abilities = {},
		spells = {}
	}
end

function player.init(attrs)
	for attr,value in pairs(attrs) do
		if value == 'max' then
			if attr == 'current_health' then
				value = player.health
			elseif attr == 'current_mana' then
				value = player.mana
			end
		end
		player[attr] = value
	end
end

function player.gettable()
	local playertable = {}
	-- simple vars
	for k,v in pairs(player) do
		if type(player[k]) == "string" or type(player[k]) == "number" then
			playertable[k] = v
		end
	end
	-- skills   
	playertable.skills = {
		paths = {},
		attributes = {},
		abilities = {},
		spells = {}
	} 
	for k,v in ipairs(player.skills.paths) do
		table.insert(playertable.skills.paths,v)
	end
	for k,v in ipairs(player.skills.attributes) do
		table.insert(playertable.skills.attributes,v)
	end
	for k,v in ipairs(player.skills.abilities) do
		table.insert(playertable.skills.abilities,v)
	end	
	for k,v in ipairs(player.skills.spells) do
		table.insert(playertable.skills.spells,v)
	end
	playertable.delayed_modifiers = {}
	for k,v in pairs(player.delayed_modifiers) do
		local modifier = {property = v.property, delay = v.delay, amount = v.amount, current_delay = v.delay}
		playertable.delayed_modifiers[k] = modifier
	end
	--[[
	playertable.cellkeys = player.cellkeys
	playertable.chestkeys = player.chestkeys
	playertable.gold = player.gold
	playertable.orbs = player.orbs
	playertable.current_health = player.current_health
	playertable.current_mana = player.current_mana
	]]
	playertable.items = {}
	for attr,item in pairs(player.items) do
		local playeritem = {}
		playeritem.tile = item.tile
		playeritem.type = item.type
	    playeritem.subtype = item.subtype
	    playeritem.name = item.name
	    playeritem.desc = item.desc
	    playeritem.value = item.value
	    playeritem.uses = item.uses
	    playeritem.count = item.count
		if item.modifier then
			playeritem.modifier = {type=item.modifier.type,property=item.modifier.property,amount=item.modifier.amount}
			if item.modifier.duration then
				playeritem.modifier.duration=item.modifier.duration
			end
			if item.modifier.max then
				playeritem.modifier.max=item.modifier.max
			end
			if item.modifier.ambi then
				playeritem.modifier.ambi=item.modifier.ambi
			end
			if item.modifier.cost then
				playeritem.modifier.cost=item.modifier.cost
			end
			if item.modifier.action then
				playeritem.modifier.action=item.modifier.action
			end
		end
		table.insert(playertable.items,playeritem)
	end
	playertable.spells = {}
	for attr,spell in pairs(player.spells) do
		local playerspell = {}
		playerspell.tile = spell.tile
		playerspell.type = spell.type
	    playerspell.subtype = spell.subtype
	    playerspell.name = spell.name
	    playerspell.desc = spell.desc
	    playerspell.value = spell.value
	    playerspell.uses = spell.uses
	    playerspell.count = spell.count
		if spell.effect then
			playerspell.effect = {type=spell.effect.type,amount=spell.effect.amount,duration=spell.effect.duration,cost=spell.effect.cost}
			if spell.effect.property then
				playerspell.effect.property=spell.effect.property
			end
			if spell.effect.action then
				playerspell.effect.action=spell.effect.action
			end
			if spell.effect.property_out then
				playerspell.effect.property_out=spell.effect.property_out
			end

			if spell.effect.amount_out then
				playerspell.effect.amount_out=spell.effect.amount_out
			end
		end
		table.insert(playertable.spells,playerspell)
	end
	return playertable
end

function player.loadtable(playertable)
	for k,v in pairs(playertable) do
		if type(playertable[k]) == "string" or type(playertable[k]) == "number" then
			player[k] = v
		end
	end
	player.skills = playertable.skills
	player.items = playertable.items
	player.spells = playertable.spells
	player.delayed_modifiers = playertable.delayed_modifiers
	for attr,card in pairs(player.items) do
		card.sprite = interface.new_sprite(
			interface.inventory_card_width, 
			interface.inventory_card_height, 
			card.tile, 
			interface.image_card_items_inventory, 
			interface.inventory_card_tileset_width, 
			interface.inventory_card_tileset_height
		)
	end
	for attr,card in pairs(player.spells) do		
		card.sprite = interface.new_sprite(
			interface.inventory_card_width, 
			interface.inventory_card_height, 
			card.tile, 
			interface.image_card_spells_inventory, 
			interface.inventory_card_tileset_width, 
			interface.inventory_card_spell_tileset_height
		)
	end
end

function player.getSheet()
	
	local path_vals = {}
	for i,val in ipairs(player.skills.paths) do
		local value = {title=lang["skill_"..val],value = nil}
		table.insert(path_vals,value)
	end
	local attribute_vals = {}
	for i,val in ipairs(player.skills.attributes) do
		local value = {title=lang["skill_"..val],value = nil}
		table.insert(attribute_vals,value)
	end	
	local ability_vals = {}
	for i,val in ipairs(player.skills.abilities) do
		local value = {title=lang["skill_"..val],value = nil}
		if skills.abilities[val].effect.property ~= "upgrade" and skills.abilities[val].effect.property ~= "health" and skills.abilities[val].effect.property ~= "mana" then
			value.value = player[skills.abilities[val].effect.property]
		else			
			value.value = 1
		end
		table.insert(ability_vals,value)
	end
	local spell_vals = {}
	for i,val in ipairs(player.skills.spells) do
		local value = {title=lang["spell_"..val],value = nil}
		table.insert(spell_vals,value)
	end	

	local sheet = {
		{
			title = lang.sheet_charateristics,
			values = {
				{title = lang.sheet_health, value = player.health},
				{title = lang.sheet_combat, value = player.combat},
				{title = lang.sheet_movement, value = player.movement},
				{title = lang.sheet_mana, value = player.mana},
				{title = lang.sheet_vision, value = player.vision},
				{title = lang.sheet_stealth, value = player.stealth}
			}
		},
		{			
			title = lang.sheet_paths,
			values = path_vals
		},
		{			
			title = lang.sheet_attributes,
			values = attribute_vals
		},
		{			
			title = lang.sheet_abilities,
			values = ability_vals
		},
		{			
			title = lang.sheet_spells,
			values = spell_vals
		}
	}
	local combat_modifier = player.modifiers.combat+player.item_modifiers.combat
	if combat_modifier ~= 0 then
		if combat_modifier > 0 then
			combat_modifier = "+"..combat_modifier
		end
		sheet[1].values[2].value = sheet[1].values[2].value.." ["..tostring(combat_modifier).."]"
	end	
	local movement_modifier = player.modifiers.movement+player.item_modifiers.movement
	if movement_modifier ~= 0 then
		if movement_modifier > 0 then
			movement_modifier = "+"..movement_modifier
		end
		sheet[1].values[3].value = sheet[1].values[3].value.." ["..tostring(movement_modifier).."]"
	end
	local vision_modifier = 0
	if player.modifiers.vision and player.item_modifiers.vision	then
		vision_modifier = player.modifiers.vision+player.item_modifiers.vision
	end
	if vision_modifier ~= 0 then
		if vision_modifier > 0 then
			vision_modifier = "+"..vision_modifier
		end
		sheet[1].values[5].value = sheet[1].values[5].value.." ["..tostring(vision_modifier).."]"
	end
	local stealth_modifier = player.modifiers.stealth+player.item_modifiers.stealth
	if stealth_modifier ~= 0 then		
		if stealth_modifier > 0 then
			stealth_modifier = "+"..combat_modifier
		end
		sheet[1].values[6].value = sheet[1].values[6].value.." ["..tostring(stealth_modifier).."]"
	end
	return sheet
end

return player