local rooms = {}
math.randomseed( os.time() )

function rooms.init()
	rooms.floor = {}
	rooms.current_room_index = 1
	rooms.current_doors = {}
	rooms.map = {}
	rooms.portals = {}
	rooms.count = 0
	rooms.exit = nil
end

function rooms.build_level()
	local exits = rooms.count_level_exits()
	local start_room = {x=rooms.current_room_location.x,y=rooms.current_room_location.y}
	while exits>0 do
		for x=1,rooms.level.size.width do
			for y=1,rooms.level.size.height do
				if rooms.map[x..y] == nil then
					-- Check adjacent rooms	
					-- North
					if y > 1 then
						local n_room_index = x .. y-1
						if n_room_index ~= exit_index and rooms.map[n_room_index] ~= nil then
							if rooms.map[n_room_index].room.doors.S[1] then
								rooms.current_room_location.x = x
								rooms.current_room_location.y = y
								local room = rooms.require_current_room()
							end
						end
					end
					-- East
					if x < rooms.level.size.width and rooms.map[x..y] == nil then
						local e_room_index = x+1 .. y
						if e_room_index ~= exit_index and rooms.map[e_room_index] ~= nil then
							if rooms.map[e_room_index].room.doors.W[1] then
								rooms.current_room_location.x = x
								rooms.current_room_location.y = y
								local room = rooms.require_current_room()
							end
						end
					end				
					-- South
					if y < rooms.level.size.height and rooms.map[x..y] == nil then
						local s_room_index = x .. y+1
						if s_room_index ~= exit_index and rooms.map[s_room_index] ~= nil then
							if rooms.map[s_room_index].room.doors.N[1] then
								rooms.current_room_location.x = x
								rooms.current_room_location.y = y
								local room = rooms.require_current_room()
							end
						end
					end
					-- West
					if x > 1 and rooms.map[x..y] == nil then
						local w_room_index = x-1 .. y
						if w_room_index ~= exit_index and rooms.map[w_room_index] ~= nil then
							if rooms.map[w_room_index].room.doors.E[1] then
								rooms.current_room_location.x = x
								rooms.current_room_location.y = y
								local room = rooms.require_current_room()
							end
						end
					end
				end
			end
		end
		rooms.current_room_location.x = start_room.x
		rooms.current_room_location.y = start_room.y
		exits = rooms.count_level_exits()
	end
end

function rooms.set_start_location()
	if rooms.level.start_type == "edge" then
		local orientation = math.random(1,4)
		if rooms.level.start_position == "center" then
			local possible_starts = {
				{x=math.ceil(rooms.level.size.width/2),y=1},
				{x=math.ceil(rooms.level.size.width/2),y=rooms.level.size.height},
				{x=1,y=math.ceil(rooms.level.size.height/2)},
				{x=rooms.level.size.width,y=math.ceil(rooms.level.size.height/2)}
			}			
			rooms.level.start_room.x = possible_starts[orientation].x
			rooms.level.start_room.y = possible_starts[orientation].y
		else
			if orientation < 3 then
				-- NS
				rooms.level.start_room.x = math.random(1,rooms.level.size.width)
				if orientation == 1 then
					-- N
					rooms.level.start_room.y = 1
				else
					-- S
					rooms.level.start_room.y = rooms.level.size.height
				end
			else
				-- WE
				rooms.level.start_room.y = math.random(1,rooms.level.size.height)
				if orientation == 3 then
					-- W
					rooms.level.start_room.x = 1
				else
					-- E
					rooms.level.start_room.x = rooms.level.size.width
				end
			end
		end
		local entrances = {
			{wall="N",tile=rooms.level.entrance.tiles.NS,start_tiles={1,2,3,4,5,6,7}},
			{wall="S",tile=rooms.level.entrance.tiles.NS,start_tiles={43,44,45,46,47,48,49}},
			{wall="W",tile=rooms.level.entrance.tiles.WE,start_tiles={7,14,21,28,35,42,49}},
			{wall="E",tile=rooms.level.entrance.tiles.WE,start_tiles={1,8,15,22,29,36,43}}
		}	
		rooms.level.entrance.wall = entrances[orientation].wall
		rooms.level.entrance.tile = entrances[orientation].tile
		if rooms.level.entrance.player_position=="center" then
			rooms.level.player_start_position = entrances[orientation].start_tiles[4]
			rooms.level.entrance.position = 4
		else
			local entrance_position = math.random(1,7)
			rooms.level.player_start_position = entrances[orientation].start_tiles[entrance_position]
			if orientation < 3 then
				-- NS
				entrance_position = 8 - entrance_position
			end
			rooms.level.entrance.position = entrance_position
		end
	end
	rooms.level.entrance.orientation = orientation
	rooms.current_room_location = {x=rooms.level.start_room.x,y=rooms.level.start_room.y}
end

function rooms.set_exit_location(exit)
   local orientation
   local exit_room_index = nil
   -- select peripheral rooms
	local explored_rooms = {}
	local unexplored_rooms = {}
	local i = 1
	helper.log("CS. "..rooms.coords_index(rooms.current_room_location))
	for room_index,room in pairs(rooms.map) do -- todo ? privilégier les salles lointaines (murs opposés au départ)
		local x = tonumber(string.sub(room_index,1,1))
      local y = tonumber(string.sub(room_index,2))
      if (x==1 or y==1 or x==rooms.level.size.width or y==rooms.level.size.height) and room_index~=rooms.coords_index(rooms.current_room_location) then
      	if room.room.visited then
				table.insert(explored_rooms,room_index)
			else
				table.insert(unexplored_rooms,room_index)
			end
			helper.log("S. "..room_index)
		end
		i = i+1
	end
	if #unexplored_rooms>0 then
		exit_room_index = unexplored_rooms[math.random(1,#unexplored_rooms)]
		helper.log("U")
		print("U : "..tostring(exit_room_index).." - "..helper.dump(unexplored_rooms))
	elseif #explored_rooms>0 then
		exit_room_index = explored_rooms[math.random(1,#explored_rooms)]
		helper.log("E")
		print("E : "..tostring(exit_room_index))
	else		
		helper.log("R")
		exit_room_index = rooms.coords_index(rooms.current_room_location)
		print("R : "..tostring(exit_room_index))
	end
	
	exit.room.x = tonumber(string.sub(exit_room_index,1,1))
   exit.room.y = tonumber(string.sub(exit_room_index,2))
   if exit.room.x == 1 then
   	-- W
   	orientation = 3
   elseif exit.room.x == rooms.level.size.width then
   	-- E
   	orientation = 4
   elseif exit.room.y == 1 then
   	-- N
   	orientation = 1
   elseif exit.room.y == rooms.level.size.height then
   	-- S
   	orientation = 2
   end
	
	local exits = {
		{wall="N",tile=exit.tiles.NS,start_tiles={1,2,3,4,5,6,7}},
		{wall="S",tile=exit.tiles.NS,start_tiles={43,44,45,46,47,48,49}},
		{wall="W",tile=exit.tiles.WE,start_tiles={7,14,21,28,35,42,49}},
		{wall="E",tile=exit.tiles.WE,start_tiles={1,8,15,22,29,36,43}}
	}	
	exit.wall = exits[orientation].wall
	exit.tile = exits[orientation].tile
	if rooms.map[exit_room_index] ~= nil then
		-- room exists
		local exit_room = rooms.map[exit_room_index].room
		exit.tile_position = rooms.set_item_position(exit_room,'exit','all',nil,nil,exits[orientation].start_tiles)
		exit.door_position = helper.index_of(exits[orientation].start_tiles, exit.tile_position)
	else
		-- new room
		if exit.door_position=="center" then
			exit.door_position = 4
		else
			local exit_position = math.random(1,7)
			-- rooms.level.player_start_position = exits[orientation].start_tiles[exit_position]
			if orientation < 3 then
				-- NS
				exit_position = 8 - exit_position
			end
			exit.door_position = exit_position
		end
	end
	exit.tile_position = exits[orientation].start_tiles[exit.door_position]
	exit.orientation = orientation	
	rooms.exit = exit
	if rooms.count_level_exits() == 0 then
		rooms.on_last_room()
	end
	return exit
end

function rooms.require_current_room(current_level,current_player_position)
   local room_index = rooms.current_room_location.x..rooms.current_room_location.y
   local room = {}
   if rooms.map[room_index]~=nil then
   		room = rooms.map[room_index].room
   		room = rooms.set_cell_lights(room)
   		room.new = false
   		if rooms.get_stats().visited == rooms.get_stats().total then
   			helper.log("#npcs : "..#room.npcs)
   			helper.log("#deck : "..#rooms.level.tile_npcs_deck)
   			if #room.npcs==0 and #rooms.level.tile_npcs_deck>0 then
   				room.npcs = rooms.set_npcs(room,1)
   			end
   		end
   else
   		start_room = "normal"
   		if room_index == rooms.level.start_room.x..rooms.level.start_room.y then
   			start_room = "start"
   		end
   		room = rooms.add_room(room_index,start_room)
   end
   return rooms.map[room_index].room
end

function rooms.enter_room(room)	
	rooms.clear_entry(room)
	room.visited = true
	if room.message then
		menu.alertmenu(room.message,"menu_continue","hint")
		room.message = nil
	end
end

function rooms.add_room(room_index,start_room,portal,room_location)
	local current_x = rooms.current_room_location.x
	local current_y = rooms.current_room_location.y
	helper.log("Salle "..room_index)
	local room = rooms.set_current_room(room_index,start_room,portal,room_location)
	room = rooms.set_cell_lights(room)
	room.new = true
	rooms.add_to_map(room,room_index)	
	if rooms.count_level_exits() == 0 then
		rooms.on_last_room()
	end
	local compass = rooms.on_create_room("create","compass")
	if compass then
		local target = {x=rooms.current_room_location.x,y=rooms.current_room_location.y}
		interface.init_compass(target)
		interface.update_compass()
	end
	local message = rooms.on_create_room("create","message")
	if message then
		room.message = message
		menu.alertmenu(message,"menu_continue","hint")
	end
	if portal then
		rooms.current_room_location.x = current_x
		rooms.current_room_location.y = current_y
	end
	rooms.count = rooms.count + 1
	return room
end

function rooms.set_current_room(room_index,room_type,portal,room_location)
	if not room_location then
		helper.log("default room location")
		room_location = rooms.current_room_location
	end
	local room = {
		mapindex = room_index,
		floor = {},
		walls = {},
		lights = {},
		doors = {},
		map_doors = {N={},S={},W={},E={}},
		cells = {},
		hall = {
			lighted = true,
			tiles = {}
		},
		items = {},
		decor = {},
		npcs = {},
		visited = false
	}	
	local current_room_location = rooms.current_room_location
	if room_location ~= rooms.current_room_location or portal or room_type == "exit"then
		room.visited = false
	end
   -- Cells
   if room_type == "start" then
    	room.doors = {N={},S={},W={},E={}}
    	helper.log(room_location.x.." - "..room_location.y)
    	if room_location.x>1 then
    		room.doors.W[1] = 4
    	end
    	if room_location.y>1 then
    		room.doors.N[1] = 4
    	end
    	if room_location.x<rooms.level.size.width then
    		room.doors.E[1] = 4
    	end
    	if room_location.y<rooms.level.size.height then
    		room.doors.S[1] = 4
    	end
	   room.hall.tiles = rooms.set_hall_tiles(room.cells)
   elseif room_type == "exit" then
   	rooms.current_room_location = room_location
   	room.cell_positions = {
			n = 0,
			s = 0,
			e = 0,
			w = 0
		}
   	room.doors = rooms.set_current_doors(room,room_type)
   	room.hall.tiles = rooms.set_hall_tiles(room.cells)
   	rooms.current_room_location = current_room_location
   else
    	local cells = rooms.set_current_cells()
	   room.cells = cells.room_cells
	   room.cell_positions = cells.cell_positions
	   room.hall.tiles = rooms.set_hall_tiles(room.cells)
	   if portal then
	   	rooms.current_room_location = room_location
	   end
		room.doors = rooms.set_current_doors(room)
	   if portal then
	   	rooms.current_room_location = current_room_location
	   end
		rooms.trigger_event()
		local event_cells = rooms.on_create_room("create","cells")
		if event_cells then
			cells = rooms.set_current_cells(event_cells)
   		room.cells = cells.room_cells
	   	room.cell_positions = cells.cell_positions
	   	room.hall.tiles = rooms.set_hall_tiles(room.cells)
	   end
	end
	-- floor
	for x = 1, tiles.rows do
		local row ={}
    	for y = 1, tiles.columns do
    		if math.random(1,10) < 10 then 
    			row[y] = "00"
    		else
    			row[y] = "0" .. math.random(1,4)
    		end
    	end
    	room.floor[x] = row
    end
    -- walls
    -- North
	row ={}
	local wall_tiles
	local wall_type
	if room_location.y == 1 then
		if rooms.cells_model.NS.walls_edge then
			wall_tiles = rooms.cells_model.NS.walls_edge.tile
			wall_type = rooms.cells_model.NS.walls_edge_type
		else
			wall_tiles = rooms.cells_model.NS.walls.tile
			wall_type = rooms.cells_model.NS.walls_type
		end
	else
		wall_tiles = rooms.cells_model.NS.walls.tile
		wall_type = rooms.cells_model.NS.walls_type
	end
	if not wall_type then
		wall_type = "random"
	end
	if wall_type == "sequence" then
		while #wall_tiles < tiles.columns do
			for x = 1, #wall_tiles do
				table.insert(wall_tiles,wall_tiles[x])
			end
		end
	end
   for y = 1, tiles.columns do
    	for z = 1, #room.doors.N do
    		if room.doors.N[z] == y then
    			row[y] = "02"
    		end
    	end
    	if room_type == "start" and rooms.level.entrance then
    		if rooms.level.entrance.wall == "N" and rooms.level.entrance.position == y then
    			row[y] = rooms.level.entrance.tile
    		end
    	end
    	if row[y] == nil then
    		if type(wall_tiles) == "string" then
    			row[y] = wall_tiles
 			else 
 				if wall_type == "sequence" then
 					row[y] = wall_tiles[y]
 				elseif wall_type == "random" then				
 					row[y] = wall_tiles[math.random(1,#wall_tiles)]
 				end
 			end
    	end
    end
    room.walls[1] = row
	-- South
	row ={}
	local wall_tiles
	if room_location.y == rooms.level.size.height then
		if rooms.cells_model.NS.walls_edge then
			wall_tiles = rooms.cells_model.NS.walls_edge.tile
			wall_type = rooms.cells_model.NS.walls_edge_type
		else
			wall_tiles = rooms.cells_model.NS.walls.tile
			wall_type = rooms.cells_model.NS.walls_type
		end
	else
		wall_tiles = rooms.cells_model.NS.walls.tile
		wall_type = rooms.cells_model.NS.walls_type
	end
	if not wall_type then
		wall_type = "random"
	end
	if wall_type == "sequence" then
		while #wall_tiles < tiles.columns do
			for x = 1, #wall_tiles do
				table.insert(wall_tiles,wall_tiles[x])
			end
		end
	end
   for y = 1, tiles.columns do
    	for z = 1, #room.doors.S do
    		if room.doors.S[z] == y then
    			row[y] = "02"
    		end
    	end
    	if room_type == "start" and rooms.level.entrance then
    		if rooms.level.entrance.wall == "S" and rooms.level.entrance.position == y then
    			row[y] = rooms.level.entrance.tile
    		end
    	end
    	if row[y] == nil then
    		if type(wall_tiles) == "string" then
    			row[y] = wall_tiles
 			else 
 				if wall_type == "sequence" then
 					row[y] = wall_tiles[y]
 				elseif wall_type == "random" then				
 					row[y] = wall_tiles[math.random(1,#wall_tiles)]
 				end
 			end
    	end
    end
    room.walls[2] = row
    -- West
   row ={}
	local wall_tiles
	if room_location.x == 1 then
		if rooms.cells_model.WE.walls_edge then
			wall_tiles = rooms.cells_model.WE.walls_edge.tile
			wall_type = rooms.cells_model.WE.walls_edge_type
		else
			wall_tiles = rooms.cells_model.WE.walls.tile
			wall_type = rooms.cells_model.WE.walls_type
		end
	else
		wall_tiles = rooms.cells_model.WE.walls.tile
		wall_type = rooms.cells_model.WE.walls_type
	end
	if not wall_type then
		wall_type = "random"
	end
	if wall_type == "sequence" then
		while #wall_tiles < tiles.columns do
			for x = 1, #wall_tiles do
				table.insert(wall_tiles,wall_tiles[x])
			end
		end
	end
	for y = 1, tiles.columns do
    	for z = 1, #room.doors.W do
    		if room.doors.W[z] == y then
    			row[y] = "12"
    		end
    	end    	
    	if room_type == "start" and rooms.level.entrance then
    		if rooms.level.entrance.wall == "W" and rooms.level.entrance.position == y then
    			row[y] = rooms.level.entrance.tile
    		end
    	end
    	if row[y] == nil then
    		if type(wall_tiles) == "string" then
    			row[y] = wall_tiles
 			else
 				if wall_type == "sequence" then
 					row[y] = wall_tiles[y]
 				elseif wall_type == "random" then				
 					row[y] = wall_tiles[math.random(1,#wall_tiles)]
 				end
 			end
    	end
    end
    room.walls[3] = row
    -- East
	row ={}
	local wall_tiles
	if room_location.x == rooms.level.size.width then
		if rooms.cells_model.WE.walls_edge then
			wall_tiles = rooms.cells_model.WE.walls_edge.tile
			wall_type = rooms.cells_model.WE.walls_edge_type
		else
			wall_tiles = rooms.cells_model.WE.walls.tile
			wall_type = rooms.cells_model.WE.walls_type
		end
	else
		wall_tiles = rooms.cells_model.WE.walls.tile
		wall_type = rooms.cells_model.WE.walls_type
	end
	if not wall_type then
		wall_type = "random"
	end
	if wall_type == "sequence" then
		while #wall_tiles < tiles.columns do
			for x = 1, #wall_tiles do
				table.insert(wall_tiles,wall_tiles[x])
			end
		end
	end
	for y = 1, tiles.columns do
    	for z = 1, #room.doors.E do
    		if room.doors.E[z] == y then
    			row[y] = "12"
    		end
    	end    	
    	if room_type == "start" and rooms.level.entrance then
    		if rooms.level.entrance.wall == "E" and rooms.level.entrance.position == y then
    			row[y] = rooms.level.entrance.tile
    		end
    	end
    	if row[y] == nil then
    		if type(wall_tiles) == "string" then
    			row[y] = wall_tiles
 			else
 				if wall_type == "sequence" then
 					row[y] = wall_tiles[y]
 				elseif wall_type == "random" then				
 					row[y] = wall_tiles[math.random(1,#wall_tiles)]
 				end
 			end
    	end
    end
    room.walls[4] = row
    if room_type ~= "start" then
	    -- Items
	   if portal == nil then
	   	local event_items = rooms.on_create_room("create","items",false)
	   	local continue = false
	   	if not event_items then
	   		-- traps
	   		event_items =  rooms.on_create_room("create","items",true)
	   		continue = true
	   	end
	   	if event_items then
				for _, item_name in pairs(event_items) do
					local tmpitem = rooms.level.special_items[item_name]
					local item = {}	    
				   for k, v in pairs (tmpitem) do
					  item[k] = v
					end
					if not item.position then
						item.position = rooms.set_item_position(room,"item",item.placeable)
					end
					if not item.position then
						helper.log("###################")		
						helper.log("Impossible de créer l'objet d'événement")
						helper.log("###################")
					else
						table.insert(room.items,item.position,item)
						helper.log("Objet d'événement créé")
					end
				end
			end
			if not event_items or continue then
				for x=1,rooms.level.items_count do
			    	if #rooms.level.tile_items_deck>0 then
					   local tmpitem = table.remove(rooms.level.tile_items_deck)
					    -- copy table
					   local item = {}	    
					   for k, v in pairs (tmpitem) do
					   		if k == "tile" and type(v) == "table" then
					   			item[k] = v[math.random(1,#v)]
					   		else
						   		item[k] = v
						   	end
						end
						local exclude = {}
						if room_type == "exit" then
							exclude = rooms.exit.tile_position
						end
						item.position = rooms.set_item_position(room,"item",item.placeable,nil,exclude)
						-- Portals
						if item.type == "Portal" then
							if rooms.level.size.width*rooms.level.size.height>rooms.count then
								-- temporary add current room to map
								-- otherwise the current room won't be taken in account during destination room set_current_doors execution
								rooms.add_to_map(room, room_index)
								local portal = rooms.build_portal(item)
								-- remove room from map after portal creation (will be added complete during add_room execution)
								rooms.map[room_index] = nil
								if not portal then
									item.position = nil
									helper.log("Portail non créé (2)")
								end
							else
								helper.log("Portail non créé (1)")
								item.position = nil
							end
						end
						if item.position then
					   	table.insert(room.items,item.position,item)
					   end
					end
				end
			end
		else
			local event_items = rooms.on_create_room("create","items")
	   	if event_items then
				for _, item_name in pairs(event_items) do
					local tmpitem = rooms.level.special_items[item_name]
					local item = {}	    
				   for k, v in pairs (tmpitem) do
					  item[k] = v
					end
					if not item.position then
						item.position = rooms.set_item_position(room,"item",item.placeable)
					end
					if not item.position then		
						helper.alert("Impossible de créer l'objet d'événement")						
					else
						table.insert(room.items,item.position,item)
						helper.log("Objet d'événement créé")
					end
				end
			end
			portal.position = rooms.set_item_position(room,"item")
			-- set portal in hall if there is no room in cells
			if portal.position == nil then
				portal.position = rooms.set_item_position(room,"item","all")
				helper.alert("Création du portail de destination dans le hall")
			end
			table.insert(rooms.portals[portal.id],portal)
			if portal.position == nil then	
				helper.alert("Impossible de créer le portail de destination")
			else
				table.insert(room.items,portal.position,portal)
			end
			levels.get_card_by_type(levels.current.tile_items_deck,"Portal",portal.id)
			room.portal_position = portal.position
		end
		-- Npcs
		local room_depth = math.abs(rooms.level.start_room.x-room_location.x)+math.abs(rooms.level.start_room.y-room_location.y)
		room.npcs = rooms.set_npcs(room,room_depth,final_room)
	end
	if not portal then
  		rooms.current_room = room
  	end
   return room
end

function rooms.build_portal(item,start_room,dest_room,start_position,dest_position)
	local current_room = rooms.current_room_location

	if not dest_room then
		dest_room_index = rooms.get_random_room("exclude")
		if dest_room_index then
			dest_room = rooms.coords_index(rooms.get_random_room("exclude"))
		else
			dest_room = nil
		end
	end

	if dest_room then
		if not start_room then
			start_room = rooms.coords_index(current_room)
		end
		if item then
			if rooms.portals[item.id] == nil then								
				rooms.portals[item.id] = {}
				item.portal_index = 1
			end
		else
			local tmpitem = rooms.level.special_items["portal"]
			item = {}	    
		   for k, v in pairs (tmpitem) do
			  item[k] = v
			end
			while rooms.portals[item.id] ~= nil do
				item.id = item.id+1
			end
			rooms.portals[item.id] = {}
			if not start_position then
				item.position = rooms.set_item_position(rooms.map[start_room].room,"item",item.placeable,nil,exclude)
			end
			table.insert(rooms.map[start_room].room.items,item.position,item)
			if start_room == rooms.coords_index(current_room) then
				tiles.change_tile(item.position,item.tile,tiles.image_items,item.layer,item.solid)
			end
		end
		item.portal_index = 1
		item.destination = {
			position = item.position,
			room = dest_room
		}
		-- create destination portal
		local portal = {
			type = "Portal",
			id = item.id,
			tile = item.tile,
			solid = item.solid,
			layer = item.layer,
			action = "teleport",
			destination = {
				position = item.position,
				room = start_room
			}
		}
		-- create destination room if it doesn't exist
		if rooms.map[dest_room] == nil then
			rooms.current_room_location = rooms.index_coords(dest_room)
			local room = rooms.add_room(dest_room,"normal",portal,destination_room)
			rooms.current_room_location = current_room
			item.destination.position = room.portal_position
			item.destination.room = room.mapindex
		else
			local room = rooms.map[dest_room].room
			portal.position = rooms.set_item_position(room,"item")
			item.destination.position = portal.position
			table.insert(rooms.portals[portal.id],portal)
			table.insert(room.items,portal.position,portal)
			room.portal_position = portal.position
		end
		return item
	else
		return false
	end
end

function rooms.set_npcs(room,max_npcs,final_room)
	local npcs = {}
	local bosses = rooms.on_create_room("create","boss")
	if bosses then
		for _, boss_name in pairs(bosses) do
			local tmpboss = rooms.level.boss[boss_name]
			local boss = {}	    
		   for k, v in pairs (tmpboss) do
			  boss[k] = v
			end
			boss.direction = path.get_random_direction()
			local placeable = nil
			if boss.placeable then
				placeable = boss.placeable
			end
			boss.position = rooms.set_item_position(room,"npc",placeable)
		   table.insert(npcs,boss)
		end
	else
		local exclude = {}
		for x=1,math.random(rooms.level.min_npcs,max_npcs) do
	    	if #rooms.level.tile_npcs_deck>0 then
			   local tmpitem = table.remove(rooms.level.tile_npcs_deck)
			   local item = {}	    
			   	for k, v in pairs (tmpitem) do
				   item[k] = v
				end
				item.direction = path.get_random_direction()
				local placeable = nil
				if item.placeable then
					placeable = item.placeable
				end
				item.position = rooms.set_item_position(room,"npc",placeable,nil,exclude)
				table.insert(exclude,item.position)
			   table.insert(npcs,item)
			end
		end
	end
	return npcs
end

function rooms.clear_entry(room)
	for _, npc in pairs( room.npcs ) do 
		if npc.position == tiles.current_player_position then
			local adjacent = path.get_adjacent(npc.position,true,false,false)
			if #adjacent > 0 then
				npc.position = adjacent[math.random(#adjacent)]
			else
				npc.position = rooms.set_item_position(room,"npc")
			end
		end
	end
end

function rooms.set_current_cells(cell_map)
	local cells = {}
	local room_cells = {}	
	local cell_positions = {
		n = 0,
		s = 0,
		e = 0,
		w = 0
	}
	if not cell_map then
		cell_map = {cells = {}, open_ratio = rooms.level.cell_open_ratio}
		if math.random(4) == 1 then
			cell_map.cells.NW = true
    		cell_positions.n = cell_positions.n + 1
    		cell_positions.w = cell_positions.w + 1
		end
		if math.random(4) == 1 then
			cell_map.cells.NE = true
    		cell_positions.n = cell_positions.n + 1
    		cell_positions.e = cell_positions.e + 1
		end
		if math.random(4) == 1 then
			cell_map.cells.SW = true
    		cell_positions.s = cell_positions.s + 1
    		cell_positions.w = cell_positions.w + 1
		end		
		if math.random(4) == 1 then
			cell_map.cells.SE = true    	
    		cell_positions.s = cell_positions.s + 1
    		cell_positions.e = cell_positions.e + 1
		end
    	if cell_positions.n == 0 and math.random(3) == 1 then
	    	cell_map.cells.N = true
	    	cell_positions.e = cell_positions.e + 1
	    	cell_positions.w = cell_positions.w + 1
	   end 
	   if cell_positions.s == 0 and math.random(3) == 1 then
	    	cell_map.cells.S = true
	    	cell_positions.e = cell_positions.e + 1
	    	cell_positions.w = cell_positions.w + 1
	   end
	   if cell_positions.e == 0 and math.random(3) == 1 then
	    	cell_map.cells.E = true
	    	cell_positions.n = cell_positions.n + 1
	    	cell_positions.s = cell_positions.s + 1
	   end
	   if cell_positions.w == 0 and math.random(3) == 1 then
	    	cell_map.cells.W = true
	    	cell_positions.n = cell_positions.n + 1
	    	cell_positions.s = cell_positions.s + 1
    	end
    	if cell_positions.n + cell_positions.s + cell_positions.e + cell_positions.w == 0 then
	    	cell_map.cells.C = true
	   end
	end
	for direction, active in pairs(cell_map.cells) do 
		if direction then
			local cell = {
		      position = direction,
		      opened = false,
		      unlocked = false,
		      lighted = false,
		      seen = false,
		      tiles = rooms.cells_model[direction].tiles,
		      walls = rooms.cells_model[direction].walls,
		      doors = rooms.cells_model[direction].doors
		  	}
			table.insert(room_cells,cell)
		end
	end
   for _, cell in pairs( room_cells) do 
    	-- Set door
    	local door_index = math.random(1,#cell.doors)
    	cell.door = cell.doors[door_index]
    	-- Door opened ?
    	local opened = math.random(math.floor(1/cell_map.open_ratio))
    	if opened == 1 then
    		cell.opened = true
    		cell.lighted = true
    	else
    		cell.opened = false
    		cell.lighted = false
    	end
    	cell.unlocked = false
    	-- Player in cell ?
    	if rooms.player_in_cell(cell) then
    		cell.lighted = true
    	end
    	-- define inner and outer walls (for fov)
    	cell.inner_walls = {}
    	cell.outer_walls = {}
    	for wall_index, wall in ipairs( cell.walls ) do
			if wall.positions then
	    		for _,tile in pairs(wall.positions) do
	    			if helper.in_array(cell.tiles,tile) then
	    				table.insert(cell.inner_walls,tile)
	    				local adjacent = path.get_adjacent(tile)
	    				for _,t in pairs (adjacent) do
	    					if not helper.in_array(cell.tiles,t) then
	    						table.insert(cell.outer_walls,t)
	    					end
	    				end
	    			else
	    				table.insert(cell.outer_walls,tile)
	    				local adjacent = path.get_adjacent(tile)
	    				for _,t in pairs (adjacent) do
	    					if helper.in_array(cell.tiles,t) then
	    						table.insert(cell.inner_walls,t)
	    					end
	    				end
	    			end
	    		end
	    	end
    	end
    end
    cells.room_cells = room_cells
    -- cell_positions ne sert que dans cette fonction ?
    cells.cell_positions = cell_positions
    return cells
end

function rooms.set_hall_tiles(cells)
	local hall_tiles = {}
	for x=1,49 do
		local in_cell = false
		for _, cell in pairs( cells) do 
			for __, tile in pairs( cell.tiles ) do
				if tile == x then
					in_cell = true
				end
			end
		end
		if not in_cell then
			table.insert(hall_tiles,x)
		end
	end
	return hall_tiles
end

function rooms.set_cell_lights(room)
	if rooms.player_in_hall(room.hall) then
		room.hall.lighted = true
		for _, cell in pairs( room.cells) do
			if not cell.opened then
				cell.lighted = false
			else
				cell.lighted = true
			end
		end
	else
		for _, cell in pairs( room.cells) do
			if not cell.opened then
				cell.lighted = false
			else
				cell.lighted = true
			end
			if rooms.player_in_cell(cell) then
				cell.lighted = true
				current_cell = cell.position
				current_opened = cell.opened
				if not cell.opened then
					room.hall.lighted = false
				end
			end
		end
		if not current_opened then
			for _, cell in pairs( room.cells) do
				if cell.position ~= current_cell then
					cell.lighted = false
				end
			end
		end
	end
	return room
end

function rooms.set_current_doors(room,room_type)
	if not room_type then
		room_type = "normal"
	end
	local adjacent_rooms = 0
	local doors = {N={},S={},W={},E={}}
	local map_limits = {N=nil,S=nil,W=nil,E=nil}
	-- check adjacent rooms	
	local n_room_index = rooms.current_room_location.x .. rooms.current_room_location.y-1
	if rooms.map[n_room_index] ~= nil then
		adjacent_rooms = adjacent_rooms + 1
		doors.N[1] = rooms.map[n_room_index].room.doors.S[1]
		if doors.N[1] == nil then
			map_limits.N = 1
		end
	end
	local s_room_index = rooms.current_room_location.x .. rooms.current_room_location.y+1
	if rooms.map[s_room_index] ~= nil then
		adjacent_rooms = adjacent_rooms + 1
		doors.S[1] = rooms.map[s_room_index].room.doors.N[1]
		if doors.S[1] == nil then
			map_limits.S = 1
		end
	end
	local w_room_index = rooms.current_room_location.x-1 .. rooms.current_room_location.y
	if rooms.map[w_room_index] ~= nil then
		adjacent_rooms = adjacent_rooms + 1
		doors.W[1] = rooms.map[w_room_index].room.doors.E[1]
		if doors.W[1] == nil then
			map_limits.W = 1
		end
	end
	local e_room_index = rooms.current_room_location.x+1 .. rooms.current_room_location.y
	if rooms.map[e_room_index] ~= nil then
		adjacent_rooms = adjacent_rooms + 1
		doors.E[1] = rooms.map[e_room_index].room.doors.W[1]
		if doors.E[1] == nil then
			map_limits.E = 1
		end
	end
	-- check map borders
	if rooms.current_room_location.y == 1 then
		map_limits.N = 1
	elseif rooms.current_room_location.y == rooms.level.size.height then
		map_limits.S = 1
	end
	if rooms.current_room_location.x == 1 then
		map_limits.W = 1
	elseif rooms.current_room_location.x == rooms.level.size.width then
		map_limits.E = 1
	end

	local room_depth = math.abs(rooms.level.start_room.x-rooms.current_room_location.x)+math.abs(rooms.level.start_room.y-rooms.current_room_location.y)
	helper.log("Profondeur : "..room_depth)

	if #doors.N == 0 and map_limits.N == nil then
		if room.cell_positions.n == 2 then
			doors.N[1] = 4
		else
			local has_door
			if room_type == "exit" then
				has_door = room_depth+1
			else
				has_door = math.random(8)
			end
			if has_door > room_depth then
				doors.N[1] = math.random(1,7)
			end
		end
	end
	if #doors.S == 0 and map_limits.S == nil then
		if room.cell_positions.s == 2 then
			doors.S[1] = 4
		else
			local has_door
			if room_type == "exit" then
				has_door = room_depth+1
			else
				has_door = math.random(8)
			end
			if has_door > room_depth then
				doors.S[1] = math.random(1,7)
			end
		end
	end
	if #doors.W == 0 and map_limits.W == nil then
		if room.cell_positions.w == 2 then
			doors.W[1] = 4
		else
			local has_door
			if room_type == "exit" then
				has_door = room_depth+1
			else
				has_door = math.random(8)
			end
			if has_door > room_depth then
				doors.W[1] = math.random(1,7)
			end
		end
	end
	if #doors.E == 0 and map_limits.E == nil then
		if room.cell_positions.e == 2 then
			doors.E[1] = 4
		else
			local has_door
			if room_type == "exit" then
				has_door = room_depth+1
			else
				has_door = math.random(8)
			end
			if has_door > room_depth then
				doors.E[1] = math.random(1,7)
			end
		end
	end

	rooms.current_doors = doors

	return doors
end

function rooms.add_to_map(room, room_index)
   if rooms.map[room_index] == nil then
   		rooms.map[room_index] = {room=room,location=rooms.current_room_location}
   end
end

function rooms.update_map(room_updated,map_location)
   local room_index = map_location.x..map_location.y
   rooms.map[room_index] = {room=room_updated,location=map_location}
end

function rooms.set_item_position(room,item_type,placeable,cell_index,exclude,include)
	if not placeable then
		if item_type=="item" then
			placeable = "cell"
		else
			placeable = "all"
		end
	end
	helper.log(item_type..' : '..placeable)
	local room_tiles = {}
	if #room.cells>0 then
		if placeable == "cell" or placeable == "all" or #room.hall.tiles < 14 then
			for _, cell in pairs( room.cells ) do 
				for __, tile in pairs( cell.tiles ) do 
					if tile ~= cell.door.cell_in and rooms.is_vacant (tile,room,exclude) and not path.is_adjacent(tile,cell.door.cell_in) then
						if not cell_index or cell_index == _ then
							table.insert(room_tiles, tile)
						end
					end
				end
			end			
			--helper.log(#room_tiles.." cells")
		end
		if placeable == "all" or placeable == "hall" then
			helper.log("-- hall")
			for _, tile in pairs( room.hall.tiles ) do 
				if rooms.is_vacant (tile,room,exclude) and helper.in_array(rooms.cells_model.safe_tiles,tile) then
					if not cell_index or cell_index == 0 then
						table.insert(room_tiles, tile)
					end
				end
			end
			--helper.log(#room_tiles.." hall")
		end			
	elseif #room.cells==0 then
		for _, tile in pairs( room.hall.tiles ) do 
			if rooms.is_vacant (tile,room,exclude) then
				table.insert(room_tiles, tile)
			end
		end
		--helper.log(#room_tiles.." no cells")
	end
	if include ~= nil then
		local tmp_tiles = {}
		for _, tile in pairs(room_tiles) do 
			if helper.in_array(include,tile) then
				table.insert(tmp_tiles,tile)
			end
		end
		room_tiles = tmp_tiles
	end
	if #room_tiles == 0 then
		-- helper.mode ="dev"
		-- helper.debug = ""
		helper.log("###################")		
		helper.log("Pas de place pour l'objet")
		helper.log("Type : "..item_type)
		helper.log("Placeable : "..tostring(placeable))
		helper.log("Cell index : "..tostring(cell_index))
		helper.log("###################")
	else
		--helper.log("Tiles libres :")
		--helper.log(helper.serialize(room_tiles))
	end
	local item_index = math.random(1,#room_tiles)
	local position = room_tiles[item_index]
	return position
end

function rooms.is_vacant(position,room,exclude)
	local exclude = exclude
	-- specific tiles
	if not exclude then
		exclude = {}
	elseif type(exclude) == "number" then
		exclude = {exclude}
	end
	-- player (first room)
	if tiles.current_player_position == nil then
		helper.add_to_table(exclude,rooms.level.player_start_position)
	end
	-- items
	for _, item in pairs( room.items ) do 
		helper.add_to_table(exclude,item.position)
	end
	-- npcs
	for _, item in pairs( room.npcs ) do 
		helper.add_to_table(exclude,item.position)
	end
	-- doors
	for direction, doors in pairs( room.doors ) do 
		for _, index in pairs(doors) do 
			door_position = rooms.door_to_floor_index(index,direction)
			helper.add_to_table(exclude,door_position)
			local adjacent = path.get_adjacent(door_position)
			for _, val in pairs( adjacent ) do 
				helper.add_to_table(exclude,val)
			end
		end
	end
	-- helper.log("Positions exclues : ",exclude)
	if helper.in_array(exclude,position) then
		return false
	else
		return true
	end
end

function rooms.player_in_cell(cell)
	for _, tile in pairs( cell.tiles ) do 
		if tile == tiles.current_player_position then
			return true
		end
	end
	return false
end

function rooms.player_in_hall(hall)
	for _, tile in pairs(hall.tiles) do 
		if tile == tiles.current_player_position then
			return true
		end
	end
	return false
end

function rooms.get_cell_tiles(position)
	local cell = rooms.get_cell_from_position(position,tiles.current_room)
	if cell == 0 then
		return tiles.current_room.hall.tiles
	else
		return tiles.current_room.cells[cell].tiles
	end
end

function rooms.get_cell_walls(cell,side)
	local wall_cells = {}
	if cell>0 then
		cell = tiles.current_room.cells[cell]
		for wall_index, wall in ipairs( cell.walls ) do
			if wall.positions then 
				for _, position in pairs(wall.positions) do
					if helper.in_array(cell.tiles,position) then
						if not side or side == "in" then
							table.insert(wall_cells,position)
						else
							for _, tile in pairs(tiles.current_room.hall.tiles) do
								if path.is_adjacent(position,tile) then
									table.insert(wall_cells,tile)
								end
							end
						end
					else
						if not side or side == "in" then
							for _, tile in pairs(cell.tiles) do
								if path.is_adjacent(position,tile) then
									table.insert(wall_cells,tile)
								end
							end
						else
							table.insert(wall_cells,position)
						end
					end
				end
			end
		end
	end
	return wall_cells
end

function rooms.get_cell_from_position(position,current_room,return_cell)
	for cell_index, cell in pairs( current_room.cells) do
		for _, tile in pairs( cell.tiles ) do 
			if tile == position then
				if return_cell == "cell" then
					return cell
				else					
					return cell_index
				end
			end
		end
	end
	return 0
end

function rooms.door_to_floor_index(index,direction)
	if direction=="N" then
		tile_index = 8-index
	elseif direction=="S" then
		tile_index = 50 - index
	elseif direction=="W" then
		tile_index = index*7
	elseif direction=="E" then
		tile_index = 1 + (index-1)*7
	end
	return tile_index
end

function rooms.get_random_room(include_current_rooms)
	local available_rooms = rooms.get_available_rooms(include_current_rooms)
	if #available_rooms > 0 then
		local room = math.random(1,#available_rooms)
		return available_rooms[room]
	else
		return false
	end
end

function rooms.get_available_rooms(include_current_rooms,exclude_visited,output)
	if not output then
		output = "coords"
	end
	local available_rooms = {}
	for x=1,rooms.level.size.width do
		for y=1,rooms.level.size.height do
			if include_current_rooms == "include" then
				if output == "index" then
			   	table.insert(available_rooms,x..y)
			   else
					table.insert(available_rooms,{x = x, y = y})
				end
			else
				local room_index = x..y
			   if rooms.map[room_index]==nil and include_current_rooms == "exclude" then
			   	if output == "index" then
			   		table.insert(available_rooms,room_index)
			   	else
			   		table.insert(available_rooms,{x = x, y = y})
			   	end
			   elseif rooms.map[room_index]~=nil and include_current_rooms == "only" then
			   	if not exclude_visited or rooms.map.visited then
				   	if output == "index" then
				   		table.insert(available_rooms,room_index)
				   	else
				   		table.insert(available_rooms,{x = x, y = y})
				   	end
				   end
			   end
			end
		end
	end
	return available_rooms
end

function rooms.get_stats()
	local stats = {
		total = 0;
		visited = 0;
	}
	for room_index,room in pairs(rooms.map) do
		stats.total = stats.total + 1;
		if room.room.visited then
			stats.visited = stats.visited + 1
		end
	end
	helper.log("Exploration : "..stats.visited.."/"..stats.total)
	return stats
end

function rooms.count_level_exits()
	local exits = 0
	local exit_index = nil
	if rooms.exit then
		exit_index = rooms.exit.room.x..rooms.exit.room.y
	end
	for x=1,rooms.level.size.width do
		for y=1,rooms.level.size.height do
			-- if empty room
			if rooms.map[x..y] == nil then
				-- Check adjacent rooms	
				-- North
				if y > 1 then
					local n_room_index = x .. y-1
					if n_room_index ~= exit_index and rooms.map[n_room_index] ~= nil then
						if rooms.map[n_room_index].room.doors.S[1] then
							exits = exits + 1
						end
					end
				end
				-- East
				if x < rooms.level.size.width then
					local e_room_index = x+1 .. y
					if e_room_index ~= exit_index and rooms.map[e_room_index] ~= nil then
						if rooms.map[e_room_index].room.doors.W[1] then
							exits = exits + 1
						end
					end
				end				
				-- South
				if y < rooms.level.size.height then
					local s_room_index = x .. y+1
					if s_room_index ~= exit_index and rooms.map[s_room_index] ~= nil then
						if rooms.map[s_room_index].room.doors.N[1] then
							exits = exits + 1
						end
					end
				end
				-- West
				if x > 1 then
					local w_room_index = x-1 .. y
					if w_room_index ~= exit_index and rooms.map[w_room_index] ~= nil then
						if rooms.map[w_room_index].room.doors.E[1] then
							exits = exits + 1
						end
					end
				end
			end
		end
	end
	return exits
end

function rooms.trigger_event()
	local level_exits = rooms.count_level_exits()
	if levels.current.events.create_room then
		for event_index, event in pairs(levels.current.events.create_room) do
			helper.log("Event : "..event_index.." : "..event.state)
			if event.state == "pending" then
				if type(event.count) == 'number' then
					if event.count == 0 then
						if level_exits == 0 then
							event.state = 'active'
						end
					elseif rooms.count == event.count or (event.main and level_exits == 0) then
						event.state = 'active'
					end
				else
					local min = event.count.min
					local max = event.count.max
					helper.log("Nbr salles "..rooms.count.."/"..min)
					if event.main and level_exits == 0 then
							event.state = 'active'
					elseif rooms.count >= min then
						local ratio = 100 / (max - rooms.count)
						roll = math.random(99)
						if roll <= ratio or level_exits == 0 then
							event.state = 'active'
						end
					end
				end
			else
				event.state = 'done'
			end
		end
	end
end

function rooms.on_create_room(action,target,traps)
	local result = false
	if traps then
		-- traps
		if action=="create" and target=="items" and levels.current.trap_ratio then
			local has_trap = math.random(math.floor(1/levels.current.trap_ratio))
			if has_trap == 1 then
				result = {}
				for i=1,math.random(levels.current.trap_count) do
					table.insert(result,"trap")
				end
			end
		end
	else
		-- event items
		if levels.current.events.create_room then
			for event_index, event in pairs(levels.current.events.create_room) do
				if event.state=='active' and event.targets[action].active[target] then
					event.targets[action].active[target] = false
					result = event.targets[action][target]
				end
			end
		end
	end
	helper.log("R:"..tostring(result))
	return result
end

function rooms.on_last_room()
	-- helper.debug = ""
	helper.log("#####################")
	helper.log("Dernière pièce")
	if rooms.exit then
		helper.log("Sortie du niveau existante")
		-- création d'un portail si pièce de sortie non visitée
		local exit_room = rooms.map[rooms.coords_index(rooms.exit.room)]
		helper.log("exit "..rooms.coords_index(rooms.exit.room))
	end
end

function rooms.coords_index(coords)
	return coords.x..coords.y
end

function rooms.index_coords(index)
	return {x=tonumber(string.sub(index,1,1)),y=tonumber(string.sub(index,2))}
end

function rooms.find_item(table_type,attr,val,return_type)
	result = {}
	for room_index,room in pairs(rooms.map) do
		for k,v in pairs(room.room[table_type]) do
			if type(v)=='table' then
				for a,b in pairs(v) do
					if a == attr and b == val then
						if return_type == "room" then
							table.insert(result,room_index)
						else
							table.insert(result,v)
						end
					end									
				end
			end
		end
	end
	return result
end

return rooms