local tiles = {}
local ppfov = require("helpers/ppfov")

function tiles.init()
	if love.graphics.getWidth()>=1920 then
		tiles.image_dir = "1920x1080"
		tiles.tile_width = 200
		tiles.tile_height = 100
		tiles.wall_width = 100
		tiles.wall_height = 200
		tiles.tileset_width = 2000
		tiles.tileset_height = 1000
		tiles.tile_creatures_width = 160
		tiles.tile_creatures_height = 160
		tiles.tileset_creatures_width = 1600
		tiles.tileset_creatures_height = 1280
		tiles.offset_x = 40
		tiles.offset_y = 380
	else
		tiles.image_dir = "1200x780"
		tiles.tile_width = 160
		tiles.tile_height = 80
		tiles.wall_width = 80
		tiles.wall_height = 160
		tiles.tileset_width = 1600
		tiles.tileset_height = 800
		tiles.tile_creatures_width = 128
		tiles.tile_creatures_height = 128
		tiles.tileset_creatures_width = 1280
		tiles.tileset_creatures_height = 1024
		tiles.offset_x = 40
		tiles.offset_y = 380
	end
	-- tiles and walls
	tiles.image_floor = love.graphics.newImage( "images/"..tiles.image_dir.."/"..rooms.level.theme.."/floor.png" )
	tiles.image_wall = love.graphics.newImage( "images/"..tiles.image_dir.."/"..rooms.level.theme.."/wall.png" )
	tiles.image_creatures = love.graphics.newImage( "images/"..tiles.image_dir.."/"..rooms.level.theme.."/creatures.png" )
	tiles.image_player = love.graphics.newImage( "images/"..tiles.image_dir.."/player.png" )
	tiles.image_items = love.graphics.newImage( "images/"..tiles.image_dir.."/"..rooms.level.theme.."/items.png" )
	tiles.image_interface = love.graphics.newImage( "images/"..tiles.image_dir.."/interface.png" )
	tiles.rows = 7
	tiles.columns = 7
	tiles.background_tiles = {}
	tiles.background_walls = {}
	tiles.front_walls = {}
	tiles.doors = {N={},S={},W={},E={}}
	tiles.current_room = {}
	tiles.sprites = {}
	tiles.cells = {}
	tiles.path = {}
	tiles.walkable = {}
	tiles.unwalkable = {}
	tiles.seen = {}
	tiles.next_player_movement = {}
	tiles.player_step = 0
	tiles.dark = 0.01
	tiles.min_light = 0.05
	tiles.walls_alpha_default = 0.7
	tiles.tile_clicked = 0
	tiles.prev_tile_overed = 0
	tiles.tile_overed = 0	
	tiles.tile_framed = 0
	tiles.tile_overed_color = interface.tile_color_default
	tiles.celldoor_overed = 0
	tiles.celldoor_clicked = 0
	tiles.door_overed = {direction = 0, index = 0}
	tiles.logtext = "log"
	tiles.time = 0
	tiles.next_move_time = 0
	tiles.next_npc_move_time = 0
	tiles.npc_attack = nil
	tiles.current_ennemy = nil
	tiles.player_movement = 0
	tiles.player_direction = "SW"
	tiles.player_paused = false
	tiles.player_appearing = false
	tiles.halo_appearing = false
	tiles.turn = nil
	tiles.prev_turn = ""
	tiles.npcs_movements = {}
	tiles.npc_step = 0
	tiles.time = 0
	tiles.debug = false
	tiles.show_sprites = false
	local current_player_position
	-- animation	
	tiles.stepcount = 20
	tiles.current_anim = {x=0,y=0,step=0,reverse=false,offset_x=0,offset_y=0}
	tiles.stepx = tiles.tile_width / tiles.stepcount
	tiles.stepy = tiles.tile_height / tiles.stepcount
end

function tiles.clear(room)
	tiles.sprites = sprites.require_current_sprites(room,tiles.current_player_position)
	tiles.background_tiles = {}
	tiles.background_walls = {}
	tiles.front_walls = {}
	tiles.doors = {N={},S={},W={},E={}}
	tiles.cells = {}
	tiles.tile_clicked = 0
	tiles.tile_overed = 0
	tiles.tile_framed = 0
	tiles.path = {}
end

function tiles.draw()
	-- Background
	love.graphics.setColor(1, 1, 1, 1)
	love.graphics.draw( interface.image_background, 0, 0, 0, love.graphics.getWidth()/interface.image_background:getWidth(),love.graphics.getHeight()/interface.image_background:getHeight())
	-- Background door tiles
	tile_overed = 0
	 for _, tile in pairs( tiles.doors.N ) do
	 		if tiles.door_overed.direction == "N" and tiles.door_overed.index > 0 then
				 if tiles.door_overed.index == _ then
						tile_overed = 1
				 else
						tile_overed = 0
				 end
			end
			tiles.draw_tile( tile, 0, tile_overed, 1 )
			if tiles.debug then
				love.graphics.print("W:"..tostring(tile.tiletype).."-"..tostring(tile.image_tile).."-"..tostring(tile.luminosity), tile.position.x+tile.width/2, tile.position.y+tile.height/2)
			end
	 end
	tile_overed = 0
	 for _, tile in pairs( tiles.doors.E ) do
	 		if tiles.door_overed.direction == "E" and tiles.door_overed.index > 0 then
				 if tiles.door_overed.index == _ then
						tile_overed = 1
				 else
						tile_overed = 0
				 end
			end
			tiles.draw_tile( tile, 0, tile_overed, 1)			
			if tiles.debug then
				love.graphics.print("E:"..tostring(tile.tiletype).."-"..tostring(tile.image_tile).."-"..tostring(tile.luminosity), tile.position.x+tile.width/2, tile.position.y+tile.height/2)
			end
	 end
	 -- Background walls
	 for _, tile in pairs( tiles.background_walls ) do
			tiles.draw_tile( tile,0,0,1 )
	 end
	 -- Background tiles
	tile_overed = 0
	tile_clicked = 0
	for i = #tiles.background_tiles, 1, -1 do
		local tile = tiles.background_tiles[i]
		local tile_index = i
		if tiles.tile_overed and tiles.tile_overed>0 then
			 if tiles.tile_overed == tile_index then
					tile_overed = 1
			 else
					tile_overed = 0
			 end
		end
		if tiles.debug then
			if helper.in_array(tiles.walkable,tile_index) then
				tile.rgb = {0.1,1,0.1}
			else
				tile.rgb = {1,1,1}
			end
			if not path.is_vacant(tile_index) then
				tile.rgb = {1,0.1,0.1}
			end
		end
		tiles.draw_tile( tile, tile_clicked, tile_overed, 1 ) 
		-- Path
		 if tiles.path[tile_index]~=nil and tiles.turn=="player" then
			 for _, sprite in pairs( tiles.path[tile_index] ) do
					tiles.draw_tile( sprite )
			 end
		end
		-- Framed
		if tiles.tile_framed == tile_index then
			local sprite_tile = tiles.background_tiles[tile_index]
			local sprite_position = {x=sprite_tile.position.x,y=sprite_tile.position.y}
			local sprite = tiles.new_tile(sprite_position,tiles.tile_width,tiles.tile_height,28,tiles.image_interface,1)
			tiles.draw_tile( sprite ) 
		end
	end
	for tile_index, tile in pairs( tiles.background_tiles ) do
		-- Cells walls
		if tiles.cells[tile_index]~=nil then
			for _, sprite in pairs( tiles.cells[tile_index] ) do
				if not sprite.hidden then
					celldoor_is_overed = 0
				 	for _, cell in ipairs( tiles.current_room.cells ) do
						if tile_index == cell.door.position and not cell.opened and tiles.celldoor_overed == cell.position then
							celldoor_is_overed = 1
							break
						end
					end
					tiles.draw_tile( sprite,0,celldoor_is_overed )
				end
			end
		end 
		-- Sprites
		if tiles.sprites[tile_index]~=nil then
			for x = 1,rooms.level.num_layers do
				if tiles.sprites[tile_index][x] ~=nil then
					tiles.draw_tile( tiles.sprites[tile_index][x] )
					-- Debug
					if helper.mode == 'dev' then
						for _, item in pairs( tiles.current_room.items ) do 
							if item.position == tile_index then
								love.graphics.print(item.type.." - solid="..tostring(item.solid), tile.position.x+tile.width/2, tile.position.y+tile.height/2)
							end
						end
						for _, item in pairs( tiles.current_room.npcs ) do 
							if item.position == tile_index then
								love.graphics.print(item.type, tile.position.x+tile.width/2, tile.position.y+tile.height/2)
							end
						end
					end
				end
			end
		end
	 end

	 -- Debug
	 -- Floor
	 for tile_index, tile in pairs( tiles.background_tiles ) do
	 	if tiles.debug then
			love.graphics.print(tile_index, tile.position.x+tile.width/2, tile.position.y+tile.height/2)
			current_cell = rooms.get_cell_from_position(tile_index,tiles.current_room)
			love.graphics.print("c"..current_cell, tile.position.x+tile.width/2, 20+tile.position.y+tile.height/2)
		end
	 end
	 -- Background walls
	 for tile_index, tile in pairs( tiles.background_walls ) do
			--love.graphics.print(tile_index, tile.position.x+tile.width/2, tile.position.y+tile.height/2)
	 end

	 -- Front walls
	 for _, tile in pairs(tiles.front_walls) do
			tiles.draw_tile( tile,0,0)
			-- Debug
			-- love.graphics.print(_, tile.position.x+tile.width/2, tile.position.y+tile.height/2)
	 end
	  -- Front door tiles
	tile_overed = 0
	for _, tile in pairs( tiles.doors.W ) do
	 		if tiles.door_overed.direction == "W" and tiles.door_overed.index > 0 then
				 if tiles.door_overed.index == _ then
						tile_overed = 1
				 else
						tile_overed = 0
				 end
			end
			tiles.draw_tile( tile, 0, tile_overed, 1 )
			tile_overed = 0
			if tiles.debug then
				love.graphics.print("W:"..tostring(tile.tiletype).."-"..tostring(tile.image_tile).."-"..tostring(tile.luminosity), tile.position.x+tile.width/2, tile.position.y+tile.height/2)
			end
	 end
	tile_overed = 0
	 for _, tile in pairs( tiles.doors.S ) do
	 		if tiles.door_overed.direction == "S" and tiles.door_overed.index > 0 then
				 if tiles.door_overed.index == _ then
						tile_overed = 1
				 else
						tile_overed = 0
				 end
			end
			tiles.draw_tile( tile, 0, tile_overed, 1 )
			if tiles.debug then
				love.graphics.print("S:"..tostring(tile.tiletype).."-"..tostring(tile.image_tile).."-"..tostring(tile.luminosity), tile.position.x+tile.width/2, tile.position.y+tile.height/2)
			end
	 end

	 love.graphics.setColor(1, 1, 1, 1)
	 if tiles.player_paused then
	 	love.graphics.print("pause",0,80)
	 else
	 	love.graphics.print("play",0,80)
	 end
end

function tiles.update( dt )
	if not menu.visible then
		tiles.time = tiles.time + dt
		local player_position = tiles.current_player_position
		local player_layer = rooms.level.special_items.player.layer
		local player_sprite = tiles.sprites[player_position][player_layer]
		-- tile overed change
		if tiles.tile_overed~=tiles.prev_tile_overed then
			tiles.set_luminosity(tiles.current_room.lights,1,5)
		end
		-- Player wounded
		if player_sprite and player_sprite.hit then
			local rgb = player_sprite.rgb
			if rgb[2]<1 then
				rgb[2]=rgb[2]+0.1
				rgb[3]=rgb[3]+0.1
				player_sprite.rgb = rgb
			else
				player_sprite.hit = false
			end
		end
		-- NPC wounded or dead
		for _, npc in pairs( tiles.current_room.npcs ) do 
			if tiles.sprites[npc.position] then
				local e_sprite = tiles.sprites[npc.position][npc.layer]
				if e_sprite and e_sprite.hit then
					local rgb = e_sprite.rgb
					if rgb[2]<1 then
						rgb[2]=rgb[2]+0.1
						rgb[3]=rgb[3]+0.1
						e_sprite.rgb = rgb
					else
						e_sprite.hit = false
					end
				end
			end
		end
		 -- Player apparition	 
		 if tiles.player_appearing then
			 local halo_layer = rooms.level.special_items.halo.layer
			 local halo_sprite = tiles.sprites[player_position][halo_layer]
			 if tiles.halo_appearing then
		 		if halo_sprite.alpha ~= nil and halo_sprite.alpha <1 then
		 			tiles.sprites[player_position][halo_layer].alpha = halo_sprite.alpha + 0.03
		 		else
		 			tiles.halo_appearing = false
		 		end
			else
			 	if halo_sprite.alpha ~= nil and halo_sprite.alpha>0 then
			 		tiles.sprites[player_position][player_layer].alpha = player_sprite.alpha + 0.03
			 		tiles.sprites[player_position][halo_layer].alpha = halo_sprite.alpha - 0.01
			 	else
			 		tiles.sprites[player_position][player_layer].alpha = 1
			 		tiles.sprites[player_position][halo_layer] = nil
			 		table.remove(tiles.current_room.items, tiles.halotoremove_index)
			 		tiles.player_appearing = false
			 		if tiles.turn == "pause" then
			 			tiles.turn = "npcs"
			 			tiles.next_turn = "npcs"
			 		else
			 			tiles.turn = "player"
			 		end
			 		if tiles.player_movement == 0 then
			 			tiles.player_movement = player.movement+player.modifiers.movement+player.item_modifiers.movement
			 		end
			 	end
			end
		end
		-- Flashing items
		for _, item in pairs( tiles.current_room.items ) do 
			if item.type == "Flashing" then
				local item_sprite = tiles.sprites[item.position][item.layer]
				if item_sprite then
					if item.fade_state == "in" then
						if item_sprite.alpha<1 then
							item_sprite.alpha = item_sprite.alpha + item.fade_in
							item.alpha = item_sprite.alpha
							tiles.set_luminosity(tiles.current_room.lights,1,5)
						else
							item.fade_state = "out"
						end
					elseif item.fade_state == "out" then
						if item_sprite.alpha>0 then
							item_sprite.alpha = item_sprite.alpha - item.fade_out
							item.alpha = item_sprite.alpha
							tiles.set_luminosity(tiles.current_room.lights,1,5)
						else
							tiles.sprites[item.position][item.layer] = nil
							table.remove(tiles.current_room.items, item.index)
						end
					end
				end
			end
		end
		 -- Player movement
		 if not tiles.player_paused 
		 	and tiles.player_step>0 
		 	and tiles.player_step <= #tiles.next_player_movement then
		 		local step = tiles.next_player_movement[tiles.player_step]
		 		local position = step.position
		 		local stepcount = tiles.stepcount
		 		if tiles.player_step > 1 then
		 			local radius = 0
		 			if step.direction == "W" or step.direction == "E" then
		 				radius = tiles.tile_width/2
		 			elseif step.direction == "N" or step.direction == "S" then
		 				radius = tiles.tile_height/2
		 			else
		 				radius = math.sqrt(math.pow(tiles.tile_width,2)+math.pow(tiles.tile_height,2))/2
		 			end
		 			local pty = (radius * math.sin(math.pi + (math.pi * tiles.current_anim.step / tiles.stepcount)));
		 			pty = math.abs(pty)/40
			 		tiles.current_anim.y = tiles.current_anim.y-pty
			 	end
		 		tiles.current_anim.reverse = false
		 		if step.direction and step.direction ~= "" then	 			
			 		tiles.player_direction = step.direction
			 		if step.direction == "SE" then	 			
			 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx
			 			tiles.current_anim.reverse = true
			 			tiles.current_anim.offset_y = 0
			 			tiles.current_anim.offset_x = tiles.tile_width
			 		elseif step.direction == "NW" then	 			
			 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx
			 		elseif step.direction == "NE" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy
			 		elseif step.direction == "SW" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy
			 			tiles.current_anim.reverse = true
			 			tiles.current_anim.offset_y = tiles.tile_height
			 			tiles.current_anim.offset_x = 0
			 		elseif step.direction == "N" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy/2
			 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx/2
			 		elseif step.direction == "S" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy/2
			 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx/2
			 			tiles.current_anim.reverse = true
			 			tiles.current_anim.offset_y = tiles.tile_height/2
			 			tiles.current_anim.offset_x = tiles.tile_width/2
			 		elseif step.direction == "W" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy/2
			 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx/2
			 			tiles.current_anim.reverse = true
			 			tiles.current_anim.offset_y = tiles.tile_height/2
			 			tiles.current_anim.offset_x = - tiles.tile_width/2
			 		elseif step.direction == "E" then	 			
			 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy/2
			 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx/2
			 		end
			 	end
		 		tiles.current_anim.step = tiles.current_anim.step +1
		 		if tiles.player_step > 1 then
			 		if tiles.current_anim.reverse then
						tiles.sprites[player_position][player_layer] = nil
			 			tiles.update_sprite(position,rooms.level.special_items.player.tile,tiles.image_player,player_layer,true,tiles.current_anim,step.direction)
			 			tiles.current_player_position = position
			 		else
			 			tiles.update_sprite(player_position,rooms.level.special_items.player.tile,tiles.image_player,player_layer,true,tiles.current_anim,step.direction)
			 		end
			 	end
				tiles.set_luminosity(tiles.current_room.lights,1,5)
				tiles.path[position]=nil
			 	if tiles.current_anim.step == stepcount then
			 		tiles.current_anim = {x=0,y=0,step=0,reverse=false,offset_x=0,offset_y=0}
					if not tiles.sprites[position] then
						tiles.sprites[position] = {}
					end
					tiles.sprites[position][player_layer] = tiles.sprites[player_position][player_layer]
					tiles.sprites[player_position][player_layer] = nil
					if tiles.player_step > 1 then
						tiles.update_sprite(position,rooms.level.special_items.player.tile,tiles.image_player,player_layer,true,nil,step.direction)
					end
					-- remove previous position and path
					tiles.current_player_position = position
					-- reset lights
					tiles.set_luminosity(tiles.current_room.lights,1,5)
					-- check player position vs items
					if tiles.player_step > 1 then
						-- play sound
						sounds.play_sound('player_step_sound',position)
						local new_action = actions.player_collision()
						if new_action~=false then
							tiles.player_movement = 0
							tiles.next_action = new_action
							tiles.next_turn = new_action
							tiles.player_step = #tiles.next_player_movement
						end				
			 			actions.npc_follow()
					end
					-- next step or stop
					if(tiles.player_step == #tiles.next_player_movement or tiles.player_movement==0) then
						-- goal reached
						if tiles.player_step == #tiles.next_player_movement then					
							tiles.next_player_movement = {}
							tiles.player_step = 0
							if tiles.next_action == "exit" then
								tiles.next_turn = "exit"
							elseif tiles.next_action == "opencell" then
								tiles.next_turn = "opencell"
							elseif tiles.next_action == "pickcell" then
								tiles.next_turn = "pickcell"				
							elseif tiles.next_action == "attack" then
								tiles.next_turn = "attack"										
							elseif tiles.next_action == "pick" then
								tiles.next_turn = "pick"
							elseif tiles.next_action == "victory" then
								tiles.next_turn = "victory"
							end
						end
						-- no movement left
						if tiles.player_movement == 0 
							and tiles.next_turn ~= "exit" 
							and tiles.next_turn ~= "opencell" 
							and tiles.next_turn ~= "victory" 
							and tiles.next_turn ~= "teleport" then					
							tiles.next_player_movement = {}
							tiles.next_turn = "npcs"
						end
					else
						tiles.player_step = tiles.player_step+1
						tiles.player_movement = tiles.player_movement -1
					end
				end
		 end
		 -- Npcs movement
		 if #tiles.npcs_movements>0 and tiles.npc_step>0 then 		 
	 		local step = tiles.npcs_movements[1].path[tiles.npc_step]
		 	local position = step.position
		 	local stepcount = tiles.stepcount
	 		if tiles.npc_step > 1 then
	 			local radius = 0
	 			if step.direction == "W" or step.direction == "E" then
	 				radius = tiles.tile_width/2
	 			elseif step.direction == "N" or step.direction == "S" then
	 				radius = tiles.tile_height/2
	 			else
	 				radius = math.sqrt(math.pow(tiles.tile_width,2)+math.pow(tiles.tile_height,2))/2
	 			end
	 			local pty = (radius * math.sin(math.pi + (math.pi * tiles.current_anim.step / tiles.stepcount)));
	 			pty = math.abs(pty)/40
	 			tiles.current_anim.y = tiles.current_anim.y - pty
	 		end
	 		tiles.current_anim.reverse = false
	 		if step.direction == "SE" then	 			
	 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx
	 			tiles.current_anim.reverse = true
	 			tiles.current_anim.offset_y = 0
	 			tiles.current_anim.offset_x = tiles.tile_width
	 		elseif step.direction == "NW" then	 			
	 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx
	 		elseif step.direction == "NE" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy
	 		elseif step.direction == "SW" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy
	 			tiles.current_anim.reverse = true
			 	tiles.current_anim.offset_y = tiles.tile_height
			 	tiles.current_anim.offset_x = 0
	 		elseif step.direction == "N" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy/2
	 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx/2
	 		elseif step.direction == "S" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy/2
	 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx/2
	 			tiles.current_anim.reverse = true
	 			tiles.current_anim.offset_y = tiles.tile_height/2
	 			tiles.current_anim.offset_x = tiles.tile_width/2
	 		elseif step.direction == "W" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y + tiles.stepy/2
	 			tiles.current_anim.x = tiles.current_anim.x - tiles.stepx/2
	 			tiles.current_anim.reverse = true
		 		tiles.current_anim.offset_y = tiles.tile_height/2
		 		tiles.current_anim.offset_x = - tiles.tile_width/2
	 		elseif step.direction == "E" then	 			
	 			tiles.current_anim.y = tiles.current_anim.y - tiles.stepy/2
	 			tiles.current_anim.x = tiles.current_anim.x + tiles.stepx/2
	 		end
		 	tiles.current_anim.step = tiles.current_anim.step +1
		 	if tiles.npc_step > 1 then
		 		if tiles.current_anim.reverse then
		 			tiles.sprites[tiles.current_npc_position][3] = nil
		 			tiles.update_sprite(position,tiles.npcs_movements[1].tile,nil,3,true,tiles.current_anim,step.direction)
		 			tiles.current_npc_position = position
		 		else
		 			tiles.update_sprite(tiles.current_npc_position,tiles.npcs_movements[1].tile,nil,3,true,tiles.current_anim,step.direction)
		 		end
		 	end	 		
			tiles.set_luminosity(tiles.current_room.lights,1,5)
		 	if tiles.current_anim.step == tiles.stepcount then	
		 		tiles.current_anim.step = 0	
		 		tiles.current_anim.x = 0
		 		tiles.current_anim.y = 0
				if tiles.current_ennemy then
					tiles.npc_attack = tiles.current_ennemy
					tiles.current_ennemy = nil
				end
			 	if tiles.npc_step == 1 then
			 		tiles.current_npc_position = tiles.npcs_movements[1].path[1].position
			 		-- print("départ",tiles.current_npc_position,tiles.npcs_movements[1].position,tiles.current_sprites[tiles.current_npc_position].type)
			 	else
				 	step = tiles.npcs_movements[1].path[tiles.npc_step]
				 	if step then
					 	local position = step.position
					 	local sprite_tile = tiles.npcs_movements[1].tile
						-- print(tiles.current_sprites[tiles.current_npc_position].type,"prev",tiles.current_npc_position,"current",position)
					 	if not tiles.sprites[position] then
							tiles.sprites[position] = {}
						end
						tiles.sprites[position][3] = tiles.sprites[tiles.current_npc_position][3]
						tiles.sprites[tiles.current_npc_position][3] = nil
						tiles.update_sprite(position,sprite_tile,nil,3,true,nil,step.direction)
						-- play sound
						sounds.play_sound('player_step_sound',position,true)
						-- reset lights
						tiles.set_luminosity(tiles.current_room.lights,1,5)
						tiles.current_npc_position = position
					end
				end
				-- next step or stop
				if tiles.npc_step == #tiles.npcs_movements[1].path or tiles.npc_step == tiles.npcs_movements[1].speed then
					-- npc attack ?
					if tiles.npc_step == #tiles.npcs_movements[1].path then
						for npc_index, npc in pairs(tiles.current_room.npcs) do
							print ("Ennemi",tiles.current_npc_position,npc.position)
							if tiles.current_npc_position == npc.position then
								tiles.current_ennemy = npc
								tiles.current_ennemy.index = npc_index
							end
						end
					else
						tiles.current_ennemy = nil
					end
					table.remove(tiles.npcs_movements,1)
					if  #tiles.npcs_movements>0 then
						tiles.npc_step = 1
					else
						if tiles.current_ennemy then
							tiles.npc_attack = tiles.current_ennemy
							tiles.current_ennemy = nil
						end
						tiles.npc_step = 0
						tiles.path = {}
						tiles.next_turn = "player"
					end			
				else
					tiles.npc_step = tiles.npc_step+1
				end
			end
		 end
	end
end

function tiles.draw_tile( single_tile, tile_clicked, tile_overed, alpha)
	 if single_tile.quad then
	 		if single_tile.alpha then
	 			alpha = single_tile.alpha
	 		end
	 		if single_tile.rgb then
	 			love.graphics.setColor(single_tile.rgb[1],single_tile.rgb[2],single_tile.rgb[3], alpha)
	 		else
	 			love.graphics.setColor(single_tile.luminosity,single_tile.luminosity,single_tile.luminosity, alpha)
	 		end
			if tile_clicked == 1 then
				 love.graphics.setColor(1, 0.9, 0, 1)
			end
			if tile_overed == 1 and tile_overed ~= tile_clicked then
				 love.graphics.setColor(tiles.tile_overed_color)
			end
			if single_tile.move_up and alpha<0.5 then
				single_tile.position.y = single_tile.position.y - single_tile.move_up
				single_tile.move_up = single_tile.move_up * 1.1
			end
			love.graphics.draw( single_tile.image_tile,
					 single_tile.quad, 
					 single_tile.position.x,
					 single_tile.position.y)
			love.graphics.setColor(1, 1, 1, 1)
	 end  
end

function tiles.tiletype_to_quad( tiletype, width, height, tileset_width, tileset_height )
	 if tiletype == nil then
			return nil
	 end
	 local row = math.floor( tiletype / 10 )
	 local col = tiletype % 10
	 local x_pos = width * ( col ) 
	 local y_pos = height * ( row )
	 local twidth = tileset_width or tiles.tileset_width
	 local theight = tileset_height or tiles.tileset_height
	 return love.graphics.newQuad( x_pos, y_pos, width, height, twidth, theight )
end

function tiles.new_tile(  position, width, height, tiletype, image_tile, luminosity, solid, direction )
	 if solid == nil then
	 	solid = true
	 end
	 if direction then
		if direction == "SW" then
			--
		elseif direction == "S" then
			tiletype = tiletype + 10
		elseif direction == "SE" then
			tiletype = tiletype + 20
		elseif direction == "E" then
			tiletype = tiletype + 30
		elseif direction == "NE" then
			tiletype = tiletype + 40
		elseif direction == "N" then
			tiletype = tiletype + 50
		elseif direction == "NW" then
			tiletype = tiletype + 60
		elseif direction == "W" then
			tiletype = tiletype + 70
		end
	end
	 if image_tile == tiles.image_creatures or image_tile == tiles.image_player then
	 	return( { position = {x=position.x+20,y=position.y+30},
			 width = tiles.tile_creatures_width,
			 height = tiles.tile_creatures_height,
				tiletype = tiletype,
				image_tile = image_tile,
				luminosity = luminosity,
				layer = layer,
				solid = solid,
				quad = tiles.tiletype_to_quad( tiletype, tiles.tile_creatures_width, tiles.tile_creatures_height, tiles.tileset_creatures_width, tiles.tileset_creatures_height)} )
	 else
	 	return( { position = position,
			 width = width or tiles.tile_width,
			 height = height or tiles.tile_height,
				tiletype = tiletype,
				image_tile = image_tile,
				luminosity = luminosity,
				layer = layer,
				solid = solid,
				quad = tiles.tiletype_to_quad( tiletype, width, height )} )
	end
end

function tiles.change_tile(position,new_tile,tileset,layer, solid, direction)
		local sprite_tile = tiles.background_tiles[position]
		local new_sprite_position = {x=sprite_tile.position.x,y=sprite_tile.position.y-tiles.tile_height}
		local new_sprite = tiles.new_tile( new_sprite_position,tiles.tile_width,tiles.tile_height*2,new_tile,tileset,tiles.dark, solid)
		if tiles.sprites[position] then
			tiles.sprites[position][layer] = {}
		end
		tiles.add_to_sprites( new_sprite, position, layer )
		tiles.update_sprite(position,new_tile,tileset,layer,solid, direction)
		tiles.set_luminosity(tiles.current_room.lights,1,5)
end

function tiles.add_to_background_tiles( tile )
	 table.insert( tiles.background_tiles, tile )
end

function tiles.add_to_background_walls( tile )
	 table.insert( tiles.background_walls, tile )
end

function tiles.add_to_north_doors( tile )
	 table.insert( tiles.doors.N, tile )
end
function tiles.add_to_south_doors( tile )
	 table.insert( tiles.doors.S, tile )
end
function tiles.add_to_east_doors( tile )
	 table.insert( tiles.doors.E, tile )
end
function tiles.add_to_west_doors( tile )
	 table.insert( tiles.doors.W, tile )
end

function tiles.add_to_front_walls( tile )
	 table.insert( tiles.front_walls, tile )
end

function tiles.add_to_sprites( sprite, tile_index, layer)
	if not tiles.sprites[tile_index] then
		tiles.sprites[tile_index] = {}
	end
	tiles.sprites[tile_index][layer] = sprite
end

function tiles.add_to_cell_walls( sprite, tile_index )
	 if tiles.cells[tile_index] == nil then
			tiles.cells[tile_index] = {}
	 end
	 table.insert( tiles.cells[tile_index], sprite )
end

function tiles.construct_level( room,sprites,level,apparition,direction)
	if direction then
		tiles.player_direction = direction
	end
	tiles.current_room = room	
	tiles.current_level = level
	tiles.screen_init()
	tiles.construct_walls(room.walls)
	tiles.construct_floor(room.floor)
	tiles.position_sprites(sprites,tiles.background_tiles)
	if not apparition then
		apparition = false
	else
		local halo = tiles.add_halo(tiles.current_player_position)
		tiles.halo_appearing = true
		tiles.player_appearing = true
		sounds.teleport:play() 
	end	
	tiles.set_luminosity(room.lights,1,5,apparition)
	if apparition then
		local player_position = tiles.current_player_position
		local player_layer = rooms.level.special_items.player.layer
	else
		tiles.turn = "player"
	end
	map.init()
  	sheet.init()
  	transition.init()
end

function tiles.screen_init()	
	local room_width = tiles.columns * tiles.tile_width
  	local room_height = tiles.rows * tiles.tile_height + tiles.wall_height
	local x = love.graphics.getWidth()/2 - room_width/2
  	local y = love.graphics.getHeight()/2
  	tiles.start_position =
  	{
  		x = x,
  		y = y
  	}
  	image_dir = interface.image_dir
  	tiles.image_floor = love.graphics.newImage( "images/"..image_dir.."/"..rooms.level.theme.."/floor.png" )
	tiles.image_wall = love.graphics.newImage( "images/"..image_dir.."/"..rooms.level.theme.."/wall.png" )
	tiles.image_creatures = love.graphics.newImage( "images/"..image_dir.."/"..rooms.level.theme.."/creatures.png" )
	tiles.image_items = love.graphics.newImage( "images/"..image_dir.."/"..rooms.level.theme.."/items.png" )
end

function tiles.construct_floor( level_tiles )
	 local current_cols = 1
	 local current_rows = tiles.rows
	 local reverse_start_x =  tiles.start_position.x+tiles.tile_width*(tiles.columns/2)-tiles.tile_width/2
	 local reverse_start_y =  tiles.start_position.y-tiles.tile_height*(tiles.columns/2)+tiles.tile_height/2
	 for row_index, row in ipairs( level_tiles ) do 
			--if row_index<=tiles.columns then
				 local current_start_x = reverse_start_x;
				 local current_start_y = reverse_start_y;
				 for col_index, tiletype in ipairs( row ) do
						if col_index<=tiles.columns then
							 local new_tile_position_x = current_start_x;
							 local new_tile_position_y = current_start_y;  
							 local new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
							 local new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,tiletype,tiles[rooms.level.tilesets.floor],tiles.dark)
							 tiles.add_to_background_tiles( new_tile )
							 current_start_x = current_start_x - tiles.tile_width/2;
							 current_start_y = current_start_y + tiles.tile_height/2;
						end
				 end
			--end
			reverse_start_x = reverse_start_x + tiles.tile_width/2;
			reverse_start_y = reverse_start_y + tiles.tile_height/2;
	 end
end

function tiles.construct_walls( level_tiles )
	local tile_width = tiles.wall_width
	local tile_height = tiles.wall_height
	 -- North wall : first row
	 local start_position = {x=tiles.start_position.x, y=tiles.start_position.y-tile_height+tiles.tile_height/2}
	 local north_wall = level_tiles[1]   
	 if tiles.current_room.exit then
	 	helper.log("=== EXIT ===")
	 	helper.log("Wall : "..tiles.current_room.exit.wall)
	 	helper.log("Door :"..tiles.current_room.exit.door_position)
	 end
	 for col_index, tiletype in ipairs( north_wall ) do
			 local new_tile_position_x = start_position.x +
					( col_index - 1 ) * tile_width
			 local new_tile_position_y = start_position.y -
					( col_index - 1 ) * tiles.tile_height/2  
			 local new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
			 local new_tile = tiles.new_tile( new_tile_position,tile_width,tile_height,tiletype,tiles[rooms.level.tilesets.wall],tiles.dark,true )
			 tiles.add_to_background_walls( new_tile )
			 -- add floor tile behind doors
			 for _, door_index in ipairs(tiles.current_room.doors.N) do
					if col_index == door_index then
							new_tile_position_x = new_tile_position_x - tiles.tile_width/2
							new_tile_position_y = new_tile_position_y + tiles.tile_height
							new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
							new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
							new_tile.adjacent = tiles.wall_to_floor_index(col_index,"back")
							tiles.add_to_north_doors( new_tile,"N" )
					end
			 end
			 -- add floor tile behind level exit
			 if tiles.current_room.exit then
			 	if tiles.current_room.exit.wall == "N" and col_index == tiles.current_room.exit.door_position then
					new_tile_position_x = new_tile_position_x - tiles.tile_width/2
					new_tile_position_y = new_tile_position_y + tiles.tile_height
					new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
					new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
					new_tile.adjacent = tiles.wall_to_floor_index(col_index,"back")
					tiles.add_to_north_doors( new_tile,"N" )
					table.insert(tiles.current_room.doors.N,col_index)
		 			helper.log("Create north exit : "..new_tile_position.x.." - "..new_tile_position.y)
				end
			 end
	 end
	 -- East wall : fourth row
	 start_position = {x=tiles.start_position.x+tile_width*tiles.columns, y=tiles.start_position.y-tile_height/2-(tiles.tile_height*tiles.rows)/2}
	 local east_wall = level_tiles[4]   
	 for col_index, tiletype in ipairs( east_wall ) do
			 local new_tile_position_x = start_position.x +
					( col_index - 1 ) * tile_width
			 local new_tile_position_y = start_position.y +
					( col_index - 1 ) * tiles.tile_height/2  
			 local new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
			 local new_tile = tiles.new_tile( new_tile_position,tile_width,tile_height,tiletype,tiles[rooms.level.tilesets.wall],tiles.dark,true )
			 tiles.add_to_background_walls( new_tile )
			 -- add floor tile behind doors
			 for _, door_index in ipairs(tiles.current_room.doors.E) do
					if col_index == door_index then
							local tile_position_x = new_tile_position_x
							local tile_position_y = new_tile_position_y + tiles.tile_height
							new_tile_position = {x= tile_position_x,y= tile_position_y}
							new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
							new_tile.adjacent = tiles.wall_to_floor_index(col_index+7,"back")
							tiles.add_to_east_doors( new_tile,"E" )
					end
			 end
			 -- add floor tile behind level exit
			 if tiles.current_room.exit then
			 	if tiles.current_room.exit.wall == "E" and col_index == tiles.current_room.exit.door_position then
			 		local tile_position_x = new_tile_position_x
					local tile_position_y = new_tile_position_y + tiles.tile_height
					new_tile_position = {x=tile_position_x,y=tile_position_y}
					new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
					new_tile.adjacent = tiles.wall_to_floor_index(col_index+7,"back")
					tiles.add_to_east_doors( new_tile,"E" )
					table.insert(tiles.current_room.doors.E,col_index)
		 			helper.log("Create east exit : "..new_tile_position.x.." - "..new_tile_position.y)
			 	end
			 end
	 end
	 -- South wall : second row
	 start_position = {x= tiles.start_position.x+tile_width*tiles.columns, y=tiles.start_position.y-tile_height/2-tile_height/4+(tiles.tile_height*tiles.rows)/2}
	 local south_wall = level_tiles[2]   
	 for col_index, tiletype in ipairs( south_wall ) do
			 local new_tile_position_x = start_position.x +
					( col_index - 1 ) * tile_width
			 local new_tile_position_y = start_position.y -
					( col_index - 1 ) * tiles.tile_height/2  
			 local new_tile_position = {x= new_tile_position_x,y=new_tile_position_y }
			 local new_tile = tiles.new_tile( new_tile_position,tile_width,tile_height,tiletype,tiles[rooms.level.tilesets.wall],tiles.dark,true )
			 tiles.add_to_front_walls( new_tile )
			  -- add floor tile behind doors
			 for _, door_index in ipairs(tiles.current_room.doors.S) do
					if col_index == door_index then
							local tile_position_x = new_tile_position_x
							local tile_position_y = new_tile_position_y + tiles.tile_height + tiles.tile_height/2
							new_tile_position = {x= tile_position_x,y= tile_position_y }
							new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
							new_tile.adjacent = tiles.wall_to_floor_index(col_index+7,"front")
							tiles.add_to_south_doors( new_tile,"S" )
					end
			 end
			 -- add floor tile behind level exit
			 if tiles.current_room.exit then
			 	if tiles.current_room.exit.wall == "S" and col_index == tiles.current_room.exit.door_position then
					local tile_position_x = new_tile_position_x
					local tile_position_y = new_tile_position_y + tiles.tile_height + tiles.tile_height/2
					new_tile_position = {x= tile_position_x,y=tile_position_y }
					new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
					new_tile.adjacent = tiles.wall_to_floor_index(col_index+7,"front")
					tiles.add_to_south_doors( new_tile,"S" )
					table.insert(tiles.current_room.doors.S,col_index)
	 				helper.log("Create south exit : "..new_tile_position.x.." - "..new_tile_position.y)
			 	end
			 end
	 end
	 -- West wall : third row
	 local start_position = {x=tiles.start_position.x, y= tiles.start_position.y-tile_height+tiles.tile_height}
	 local west_wall = level_tiles[3]   
	 for col_index, tiletype in ipairs( west_wall ) do
			 local new_tile_position_x = start_position.x +
					( col_index - 1 ) * tile_width
			 local new_tile_position_y = start_position.y +
					( col_index - 1 ) * tiles.tile_height/2  
			 local new_tile_position = {x=new_tile_position_x,y=new_tile_position_y}
			 local new_tile = tiles.new_tile( new_tile_position,tile_width,tile_height,tiletype,tiles[rooms.level.tilesets.wall],tiles.dark,true )
			 tiles.add_to_front_walls( new_tile )
			  -- add floor tile behind doors
			 for _, door_index in ipairs(tiles.current_room.doors.W) do
					if col_index == door_index then
							local tile_position_x = new_tile_position_x - tiles.tile_width/2
							local tile_position_y = new_tile_position_y + tiles.tile_height + tiles.tile_height/2
							new_tile_position = {x=tile_position_x,y=tile_position_y}
							new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
							new_tile.adjacent = tiles.wall_to_floor_index(col_index,"front")
							tiles.add_to_west_doors( new_tile,"W" )
					end
			 end
			 -- add floor tile behind level exit
			 if tiles.current_room.exit then				 	
			 	if tiles.current_room.exit.wall == "W" and col_index == tiles.current_room.exit.door_position then
			 		local tile_position_x = new_tile_position_x - tiles.tile_width/2
					local tile_position_y = new_tile_position_y + tiles.tile_height + tiles.tile_height/2
					new_tile_position = {x=tile_position_x,y=tile_position_y}
					new_tile = tiles.new_tile( new_tile_position,tiles.tile_width,tiles.tile_height,"00",tiles[rooms.level.tilesets.floor],tiles.dark,false)
					new_tile.adjacent = tiles.wall_to_floor_index(col_index,"front")
					tiles.add_to_west_doors( new_tile,"W" )
					table.insert(tiles.current_room.doors.W,col_index)
		 			helper.log("Create west exit : "..new_tile_position.x.." - "..new_tile_position.y)
			 	end
			 end
	 end
end

function tiles.position_sprites( sprites, level_tiles, selection)
	-- sprites
	if selection == nil or selection == 'sprites' then
		 for sprite_index, layer in pairs( sprites ) do 
		 	for layer_index, sprite in pairs(layer) do
				 local sprite_tile = level_tiles[sprite_index]
				 local new_sprite_position = {x=sprite_tile.position.x,y=sprite_tile.position.y-tiles.tile_height}
				 local current_sprite_tile = sprite.tile
				 if sprite.state ~= nil then
					current_sprite_tile = sprite["tile_"..sprite.state]
				 end
				 local new_sprite = tiles.new_tile( new_sprite_position,tiles.tile_width,tiles.tile_height*2,current_sprite_tile,tiles[sprite.tileset],tiles.dark,sprite.solid,sprite.direction)
				 new_sprite.props = sprite
				 tiles.add_to_sprites( new_sprite, sprite_index, layer_index)
			end
		 end
	end
	 -- cells walls
	 if selection == nil or selection == 'cells' then
		 for _, cell in pairs( tiles.current_room.cells ) do 
		 	for wall_index, wall in ipairs( cell.walls ) do
		 		if wall.positions then 
		 			for _, position in pairs(wall.positions) do
						local sprite_tile = level_tiles[position]
						local tile = wall.tile
						if type(tile) ~= "string" then
			 				tile = tile[math.random(1,#tile)]
			 			end
						local new_sprite_position = {x=sprite_tile.position.x,y=sprite_tile.position.y-tiles.tile_height}
						if wall_index == 1 or wall_index == 3 then
							new_sprite_position.y = new_sprite_position.y - tiles.tile_height/2
						else					
							new_sprite_position.y = new_sprite_position.y - tiles.tile_height/2					
							new_sprite_position.x = new_sprite_position.x + tiles.tile_width/2
						end
						-- cell door
						if cell.door.position == position then
							if cell.opened then
								if cell.unlocked then
									tile = cell.door.tile_unlocked
								else
									tile = cell.door.tile_opened
								end
							else
								tile = cell.door.tile_closed	
							end					
						end
						local new_sprite = tiles.new_tile( new_sprite_position,tiles.wall_width,tiles.wall_height,tile,tiles[rooms.level.tilesets.wall],tiles.dark)
						tiles.add_to_cell_walls( new_sprite, position, true )
					end
				end
		 	end
		 end
	 end
end

function tiles.set_luminosity(lights,luminosity,halo,apparition)

	local lightstrength = 1	
	local player_vision = player.vision+player.modifiers.vision+player.item_modifiers.vision
	local matrixes = {
		{0.9,0.95,0.96,0.97,0.98,0.99,1,1,1,1,1,1,1},
		{0.85,0.88,0.91,0.95,0.97,0.98,0.99,1,1,1,1,1,1},
		{0.8,0.83,0.85,0.9,0.92,0.94,0.95,0.96,0.97,0.98,0.98,0.99,0.99},
		{0.7,0.75,0.8,0.88,0.92,0.94,0.95,0.96,0.97,0.98,0.98,0.99,0.99},
		{0.65,0.7,0.75,0.85,0.90,0.92,0.94,0.95,0.96,0.97,0.98,0.99,0.99},
		{0.60,0.65,0.70,0.80,0.85,0.90,0.90,0.92,0.94,0.95,0.96,0.97,0.98},
		{0.55,0.60,0.65,0.75,0.80,0.85,0.90,0.92,0.94,0.95,0.96,0.97,0.98},
		{0.50,0.55,0.60,0.65,0.70,0.75,0.80,0.85,0.90,0.95,0.96,0.97,0.98},
		{0.50,0.55,0.60,0.65,0.70,0.75,0.80,0.85,0.90,0.95,0.96,0.97,0.98},
		{0.50,0.55,0.60,0.65,0.70,0.75,0.80,0.85,0.90,0.95,0.96,0.97,0.98}
	}
	local matrix = matrixes[player_vision]
	-- uncomment to disable lighting
	-- matrix = {0,0,0,0,0,0,0,0,0,0,0,0,0}
	lights[#lights] = tiles.current_player_position
	-- darken/reset all tiles
	for tile_index, tile in pairs( tiles.background_tiles ) do 
		tiles.background_tiles[tile_index].luminosity = tiles.dark
		tiles.background_tiles[tile_index].visible = 0
		tiles.background_tiles[tile_index].is_wall = false
	end
	for sprite_index, sprite in pairs(tiles.sprites) do
		for single_sprite_index, single_sprite in pairs(tiles.sprites[sprite_index]) do
			tiles.sprites[sprite_index][single_sprite_index].luminosity = tiles.dark
		end
	end
	-- fov
	local pp = path.get_coordinates(tiles.current_player_position)
	local angle = path.direction_to_angle(tiles.player_direction)
	if player_vision<2 then
		player_vision = 2
	end
	local sight = 90+player_vision*10
	local angle_start = math.rad(angle-sight/2)
	local angle_end = math.rad(angle+sight/2)
	local permissiveness = 5
	tiles.seen = {}
	ppfov(pp.x,pp.y,6,path.isTransparent,path.onVisible,angle_start,angle_end,permissiveness)
	angle_start = math.rad(angle-sight/2)
	angle_end = math.rad(angle+sight/2)	
	tiles.seen = {}
	ppfov(pp.x,pp.y,6,path.isTransparent,path.onVisible,angle_start,angle_end,permissiveness)
	angle_start = math.rad(angle-sight/2)
	angle_end = math.rad(angle+sight/2)	
	tiles.seen = {}
	ppfov(pp.x,pp.y,6,path.isTransparent,path.onVisible,angle_start,angle_end,permissiveness)
	
	-- lighten floor tiles
	for tile_index, tile in pairs( tiles.background_tiles ) do 
			for _, light_index in pairs( lights ) do 
					local distance = tiles.get_distance(tile_index,light_index)+1  
					lightstrength = luminosity-matrix[distance]
					tiles.background_tiles[tile_index].luminosity = tiles.background_tiles[tile_index].luminosity+lightstrength
					if tiles.background_tiles[tile_index].visible>0 and not tiles.background_tiles[tile_index].is_wall then
						tiles.background_tiles[tile_index].luminosity = tiles.background_tiles[tile_index].luminosity*(tiles.background_tiles[tile_index].visible+player_vision-4)
						-- tiles.background_tiles[tile_index].luminosity = 1
					elseif not tiles.background_tiles[tile_index].is_wall then
						-- tiles.background_tiles[tile_index].luminosity = tiles.background_tiles[tile_index].luminosity*1.2
					else
						-- tiles.background_tiles[tile_index].luminosity =  0
					end
			end
			-- Flashing items
			for _, item in pairs( tiles.current_room.items ) do 
				if item.type=="Flashing" then
					if item.position == tile_index then
						helper.log("Lum av. : "..tiles.background_tiles[tile_index].luminosity)
						tiles.background_tiles[tile_index].luminosity = tiles.background_tiles[tile_index].luminosity+item.alpha
						helper.log("Lum ap. : "..tiles.background_tiles[tile_index].luminosity)
					end
				end
			end
	end	
	-- re-darken hidden tiles
	for _, cell in pairs( tiles.current_room.cells ) do 
	 	if not cell.lighted then
		 	for _, tile_index in ipairs( cell.tiles ) do 
		 		tiles.background_tiles[tile_index].luminosity = tiles.dark
		 	end
		 end
	end
	if not tiles.current_room.hall.lighted then
	 	for _, tile_index in ipairs( tiles.current_room.hall.tiles ) do 
		 	tiles.background_tiles[tile_index].luminosity = tiles.dark
		end
	end	
	-- lighten walls
	-- Background walls
	 for _, tile in pairs( tiles.background_walls ) do
			wall_tile = tiles.wall_to_floor_index(_,"back")
			tiles.background_walls[_].luminosity = tiles.background_tiles[wall_tile].luminosity
			if _<8 then
				for d, door_index in pairs(tiles.current_room.doors.N) do
					if door_index == _ then
						tiles.doors.N[d].luminosity = tiles.background_tiles[wall_tile].luminosity/2
						if tiles.doors.N[d].luminosity > tiles.min_light and tiles.current_room.map_doors.N[d] == nil then
							table.insert(tiles.current_room.map_doors.N,d,door_index)
						end
					end
				end
			else
				for d, door_index in pairs(tiles.current_room.doors.E) do
					if door_index+7 == _ then
						tiles.doors.E[d].luminosity = tiles.background_tiles[wall_tile].luminosity/2
						if tiles.doors.E[d].luminosity > tiles.min_light and tiles.current_room.map_doors.E[d] == nil then
							table.insert(tiles.current_room.map_doors.E,d,door_index)
						end
					end
				end
			end
	 end 
	-- Front walls
	 for _, tile in pairs( tiles.front_walls ) do
			if _<8 then
				wall_tile = tiles.wall_to_floor_index(_,"front")
				for d, door_index in pairs(tiles.current_room.doors.W) do
					if door_index == _ then
						tiles.doors.W[d].luminosity = tiles.background_tiles[wall_tile].luminosity/2
						if tiles.doors.W[d].luminosity > tiles.min_light and tiles.current_room.map_doors.W[d] == nil then
							table.insert(tiles.current_room.map_doors.W,d,door_index)
						end
					end
				end
				wall_tile = tiles.wall_to_floor_index(_+7,"front")
			else
				wall_tile = tiles.wall_to_floor_index(_,"front")
				for d, door_index in pairs(tiles.current_room.doors.S) do
					if door_index+7 == _ then
						tiles.doors.S[d].luminosity = tiles.background_tiles[wall_tile].luminosity/2
						if tiles.doors.S[d].luminosity > tiles.min_light and tiles.current_room.map_doors.S[d] == nil then
							table.insert(tiles.current_room.map_doors.S,d,door_index)
						end
					end
				end
				wall_tile = tiles.wall_to_floor_index(_-7,"front")
			end
			tile.luminosity = tiles.background_tiles[wall_tile].luminosity			
			-- hide dark wall tiles
			if tile.luminosity == tiles.dark then
				-- ne fonctionne pas bien, faire uniquement pour les tiles cachées
				tile.alpha = 0
			else
		 		tile.alpha = tiles.get_front_wall_alpha(_)
		 	end
	 end 
	-- cells
	local current_cell = rooms.get_cell_from_position(tiles.current_player_position,tiles.current_room,"cell")
	for sprite_index, sprite in pairs(tiles.cells) do
		for single_sprite_index, single_sprite in pairs(tiles.cells[sprite_index]) do
			local sprite_alpha = single_sprite.alpha
			single_sprite.luminosity = tiles.background_tiles[sprite_index].luminosity
			single_sprite.hidden = false
			for cell_index, cell in ipairs( tiles.current_room.cells ) do
				-- show cell on map
				if sprite_index == cell.door.position and single_sprite.luminosity>tiles.min_light then
					cell.seen = true
				end
				-- hide walls in invisible cells
				single_sprite.alpha = tiles.get_cell_wall_alpha(sprite_index)
				if current_cell~=0 then
					if current_cell.opened==false and current_cell.position~=0 and current_cell.position~=cell.position then
						for wall_index, wall in ipairs( cell.walls ) do
							if wall.positions then
								for position_index, wall_position in ipairs( wall.positions ) do
									if sprite_index == wall_position then
										single_sprite.hidden = true
									end
								end
							end
						end
					end
				end
			end
		end
	end
	-- sprites
	for sprite_index, sprite in pairs(tiles.sprites) do
		for single_sprite_index, single_sprite in pairs(tiles.sprites[sprite_index]) do			
			if tiles.show_sprites then
				single_sprite.luminosity = 1
			else
				single_sprite.luminosity = tiles.background_tiles[sprite_index].luminosity
				if apparition then
					local player_layer = rooms.level.special_items.player.layer
					local halo_layer = rooms.level.special_items.halo.layer
					local player_position = tiles.current_player_position
					if sprite_index == player_position and (single_sprite_index == player_layer or single_sprite_index == halo_layer) then
						single_sprite.alpha = 0
					end
				end
				if tiles.background_tiles[sprite_index].luminosity == tiles.dark then
					single_sprite.alpha = 0
					single_sprite.hidden = true
				elseif not single_sprite.alpha or single_sprite.hidden then
					single_sprite.alpha = 1
					single_sprite.hidden = false
				end
			end
		end
	end
end

function tiles.move_player()		
end

function tiles.update_sprite(position,sprite_type,sprite_image,layer,solid,movement,direction,alpha)
		if not sprite_image then
			sprite_image = tiles.image_creatures
		end
		local sprite_tile = tiles.background_tiles[position]
		local sprite_tile_position = {x = sprite_tile.position.x, y = sprite_tile.position.y}
		if movement then
			if movement.reverse then
				sprite_tile_position.x = sprite_tile_position.x + movement.x - movement.offset_x
				sprite_tile_position.y = sprite_tile_position.y + movement.y - movement.offset_y
			else				
				sprite_tile_position.x = sprite_tile_position.x + movement.x
				sprite_tile_position.y = sprite_tile_position.y + movement.y
			end
		end
		local new_sprite_position = {x=sprite_tile_position.x,y=sprite_tile_position.y-tiles.tile_height}
		local new_sprite = tiles.new_tile( new_sprite_position,tiles.tile_width,tiles.tile_height*2,sprite_type,sprite_image,tiles.dark,solid,direction)
		if alpha then
			new_sprite.alpha = alpha
		end
		tiles.add_to_sprites( new_sprite, position, layer)
end

function tiles.add_halo(halo_position)
	local halo = {
		  type="Halo",
	      tile=rooms.level.special_items.halo.tile,
	      solid=rooms.level.special_items.halo.solid,
	      layer=rooms.level.special_items.halo.layer
		}
	halo.position = halo_position
	table.insert(tiles.current_room.items,halo)
	tiles.halotoremove_index = #tiles.current_room.items
	tiles.change_tile(halo.position,halo.tile,tiles.image_items,halo.layer,halo.solid)
end


function tiles.add_flashing_item(item_name,item_position,fade_in,fade_out)
	if not fade_in then
		fade_in = 0.1
	end
	if not fade_out then
		fade_out = 0.05
	end
	local item = {
		  type = "Flashing",
	      tile = rooms.level.special_items[item_name].tile,
	      solid = rooms.level.special_items[item_name].solid,
	      layer = rooms.level.special_items[item_name].layer,
	      alpha = 0,
	      position = item_position,
	      fade_in = fade_in,
	      fade_out = fade_out,
	      fade_state = "in"
		}
	table.insert(tiles.current_room.items,item)
	item.index = #tiles.current_room.items
	tiles.change_tile(item.position,item.tile,tiles.image_items,item.layer,item.solid)
	local item_sprite = tiles.sprites[item.position][item.layer]
	item.alpha = 0
end

function tiles.get_distance(tile1,tile2)
	local coords1 = path.get_coordinates(tile1)
	local coords2 = path.get_coordinates(tile2)
	local distance_x = math.abs(coords1.x-coords2.x)
	local distance_y = math.abs(coords1.y-coords2.y)
	return distance_x+distance_y
end

function tiles.get_tile(tile_index)
	for index, tile in ipairs(tiles.background_tiles) do 
		if tile_index == index then
			return tile
		end
	end
	return false
end

function tiles.get_tile_items(tile_index)
	local items = {items={},npcs={}}
	for _, item in pairs( tiles.current_room.items ) do 
		if item.position == tile_index then
			table.insert(items.items,item)
		end
	end
	for _, item in pairs( tiles.current_room.npcs ) do 
		if item.position == tile_index then
			table.insert(items.npcs,item)
		end
	end
	return items
end

function tiles.get_tile_from_coords(pos_x,pos_y)
	return (pos_y-1)*tiles.columns + pos_x
end

function tiles.set_unwalkable(tile_index)
	for _, tile in pairs(tiles.walkable) do
		if tile==tile_index then
			table.remove(tiles.walkable,_)
		end
	end
end

function tiles.wall_to_floor_index(index,wall_type)
	if wall_type == "back" then
		if index < 8 then
			tile_index = 8-index
		else
			tile_index = 1 + (index-8)*7
		end
	elseif wall_type == "front" then
		if index < 8 then
			tile_index = index*7
		else
			tile_index = 57 - index
		end
	end
	return tile_index
end

function tiles.get_front_wall_alpha(wall_index)
	local alpha = tiles.walls_alpha_default
	local alpha2 = alpha
	local alpha3 = alpha
	local x = tiles.rows
	-- mouse over
	if tiles.tile_overed and tiles.tile_overed>0 then
		local t = tiles.tile_overed
		-- west wall
		if t % x*x == 0 then
			if wall_index > x and wall_index <= x*2 then
				local wall_overed = x+t/x
				alpha = 0.3 + math.abs(wall_overed-wall_index)/10
			end
		end
		-- south wall
		if t > x*(x-1) then
			if wall_index <= x then
				local wall_overed = x*x+1-t
				alpha = 0.3 + math.abs(wall_overed-wall_index)/10
			end
		end
	end
	--player position
	local t = tiles.current_player_position
	-- west wall
	if t % x*x == 0 then
		if wall_index > x and wall_index <= x*2 then
			local wall_overed = x+t/x
			alpha2 = 0.5 + math.abs(wall_overed-wall_index)/20
		end
	end
	-- south wall
	if t > x*(x-1) then
		if wall_index <= x then
			local wall_overed = x*x+1-t
			alpha2 = 0.5 + math.abs(wall_overed-wall_index)/20
		end
	end
	-- npcs
	for _, npc in pairs( tiles.current_room.npcs ) do 
		-- applies only if npc and player are in the same cell
		local player_cell  = rooms.get_cell_from_position(t,tiles.current_room)
		local npc_cell  = rooms.get_cell_from_position(npc.position,tiles.current_room)
		if player_cell == npc_cell then
			local t = npc.position
			if t then
				-- west wall
				if t % x*x == 0 then
					if wall_index > x and wall_index <= x*2 then
							local wall_overed = x+t/x
							alpha3 = 0.5 + math.abs(wall_overed-wall_index)/20
					end
				end
				-- south wall
				if t > x*(x-1) then
					if wall_index <= x then
						local wall_overed = x*x+1-t
						alpha3 = 0.5 + math.abs(wall_overed-wall_index)/20
					end
				end
			end
		end
	end
	alpha = math.min(alpha,alpha2,alpha3)
	return alpha
end

function tiles.get_cell_wall_alpha(tile_index)
	local alpha = tiles.walls_alpha_default
	local alpha2 = alpha
	local alpha3 = alpha
	-- mouse over
	if tiles.tile_overed and tiles.tile_overed>0 then
		-- get closer tiles index on both axis
		local ns_index = path.get_ahead_tile(tiles.tile_overed,"S")
		local ew_index = path.get_ahead_tile(tiles.tile_overed,"W")
		-- find walls matching the tile indexes
		local walls = {}
		for _, cell in pairs( tiles.current_room.cells ) do 
			for wall_index, wall in ipairs( cell.walls ) do
				if wall.positions then 
					for _, position in pairs(wall.positions) do
						if ns_index == position or ew_index == position then
							local matching_wall = {index = _,tiles = wall.positions}
							table.insert(walls,matching_wall)
						end
					end
				end
			end
		end
		-- set alpha according to tile proximity
		for _, wall in pairs( walls ) do
			for wall_index, tile in pairs( wall.tiles ) do
				if tile_index == tile then
					alpha = 0.3 + math.abs(wall.index-wall_index)/10
				end
			end
		end
	end
	-- player position
	-- get closer tiles index on both axis
	local ns_index = path.get_ahead_tile(tiles.current_player_position,"S")
	local ew_index = path.get_ahead_tile(tiles.current_player_position,"W")
	-- find walls matching the tile indexes
	local walls = {}
	for _, cell in pairs( tiles.current_room.cells ) do 
		for wall_index, wall in ipairs( cell.walls ) do
			if wall.positions then 
				for _, position in pairs(wall.positions) do
					if ns_index == position or ew_index == position then
						local matching_wall = {index = _,tiles = wall.positions}
						table.insert(walls,matching_wall)
					end
				end
			end
		end
	end
	-- set alpha according to tile proximity
	for _, wall in pairs( walls ) do
		for wall_index, tile in pairs( wall.tiles ) do
			if tile_index == tile then
				alpha2 = 0.5 + math.abs(wall.index-wall_index)/20
			end
		end
	end
	-- npcs
	for _, npc in pairs( tiles.current_room.npcs ) do 
		-- applies only if npc and player are in the same cell
		local player_cell  = rooms.get_cell_from_position(t,tiles.current_room)
		local npc_cell  = rooms.get_cell_from_position(npc.position,tiles.current_room)
		if player_cell == npc_cell then
			-- get closer tiles index on both axis
			local ns_index = path.get_ahead_tile(npc.position,"S")
			local ew_index = path.get_ahead_tile(npc.position,"W")
			-- find walls matching the tile indexes
			local walls = {}
			for _, cell in pairs( tiles.current_room.cells ) do 
				for wall_index, wall in ipairs( cell.walls ) do
					if wall.positions then 
						for _, position in pairs(wall.positions) do
							if ns_index == position or ew_index == position then
								local matching_wall = {index = _,tiles = wall.positions}
								table.insert(walls,matching_wall)
							end
						end
					end
				end
			end
			-- set alpha according to tile proximity
			for _, wall in pairs( walls ) do
				for wall_index, tile in pairs( wall.tiles ) do
					if tile_index == tile then
						alpha3 = 0.5 + math.abs(wall.index-wall_index)/20
					end
				end
			end
		end
	end
	alpha = math.min(alpha,alpha2,alpha3)
	return alpha
end

return tiles